@@ -1,11 +1,16 @@
 // See README.txt for information and build instructions.
 
-#include <iostream>
 #include <fstream>
+#include <google/protobuf/util/time_util.h>
+#include <iostream>
 #include <string>
+
 #include "addressbook.pb.h"
+
 using namespace std;
 
+using google::protobuf::util::TimeUtil;
+
 // Iterates though all people in the AddressBook and prints info about them.
 void ListPeople(const tutorial::AddressBook& address_book) {
   for (int i = 0; i < address_book.people_size(); i++) {
@@ -30,9 +35,15 @@ void ListPeople(const tutorial::AddressBook& address_book) {
         case tutorial::Person::WORK:
           cout << "  Work phone #: ";
           break;
+        default:
+          cout << "  Unknown phone #: ";
+          break;
       }
       cout << phone_number.number() << endl;
     }
+    if (person.has_last_updated()) {
+      cout << "  Updated: " << TimeUtil::ToString(person.last_updated()) << endl;
+    }
   }
 }
 
