@@ -57,15 +57,14 @@ public final class MapFieldLite<K, V> extends LinkedHashMap<K, V> {
     this.isMutable = true;
   }
 
-  @SuppressWarnings({"rawtypes", "unchecked"})
-  private static final MapFieldLite EMPTY_MAP_FIELD = new MapFieldLite<>();
+  private static final MapFieldLite<?, ?> EMPTY_MAP_FIELD = new MapFieldLite<>();
 
   static {
     EMPTY_MAP_FIELD.makeImmutable();
   }
 
-  /** Returns an singleton immutable empty MapFieldLite instance. */
-  @SuppressWarnings({"unchecked", "cast"})
+  /** Returns a singleton immutable empty MapFieldLite instance. */
+  @SuppressWarnings("unchecked")
   public static <K, V> MapFieldLite<K, V> emptyMapField() {
     return (MapFieldLite<K, V>) EMPTY_MAP_FIELD;
   }
@@ -77,7 +76,6 @@ public final class MapFieldLite<K, V> extends LinkedHashMap<K, V> {
     }
   }
 
-  @SuppressWarnings({"unchecked", "cast"})
   @Override
   public Set<Map.Entry<K, V>> entrySet() {
     return isEmpty() ? Collections.<Map.Entry<K, V>>emptySet() : super.entrySet();
