@@ -54,7 +54,7 @@ typedef struct {
   zend_object std;
   zval arena;
   const Descriptor* desc;
-  upb_msg *msg;
+  upb_Message *msg;
 } Message;
 
 zend_class_entry *message_ce;
@@ -110,10 +110,10 @@ static void Message_dtor(zend_object* obj) {
  *
  * Helper function to look up a field given a member name (as a string).
  */
-static const upb_fielddef *get_field(Message *msg, PROTO_STR *member) {
-  const upb_msgdef *m = msg->desc->msgdef;
-  const upb_fielddef *f =
-      upb_msgdef_ntof(m, PROTO_STRVAL_P(member), PROTO_STRLEN_P(member));
+static const upb_FieldDef *get_field(Message *msg, PROTO_STR *member) {
+  const upb_MessageDef *m = msg->desc->msgdef;
+  const upb_FieldDef *f =
+      upb_MessageDef_FindFieldByNameWithSize(m, PROTO_STRVAL_P(member), PROTO_STRLEN_P(member));
 
   if (!f) {
     zend_throw_exception_ex(NULL, 0, "No such property %s.",
@@ -123,65 +123,91 @@ static const upb_fielddef *get_field(Message *msg, PROTO_STR *member) {
   return f;
 }
 
-static void Message_get(Message *intern, const upb_fielddef *f, zval *rv) {
-  upb_arena *arena = Arena_Get(&intern->arena);
+// Check if the field is a well known wrapper type
+static bool IsWrapper(const upb_MessageDef* m) {
+  if (!m) return false;
+  switch (upb_MessageDef_WellKnownType(m)) {
+    case kUpb_WellKnown_DoubleValue:
+    case kUpb_WellKnown_FloatValue:
+    case kUpb_WellKnown_Int64Value:
+    case kUpb_WellKnown_UInt64Value:
+    case kUpb_WellKnown_Int32Value:
+    case kUpb_WellKnown_UInt32Value:
+    case kUpb_WellKnown_StringValue:
+    case kUpb_WellKnown_BytesValue:
+    case kUpb_WellKnown_BoolValue:
+      return true;
+    default:
+      return false;
+  }
+}
 
-  if (upb_fielddef_ismap(f)) {
-    upb_mutmsgval msgval = upb_msg_mutable(intern->msg, f, arena);
+static void Message_get(Message *intern, const upb_FieldDef *f, zval *rv) {
+  upb_Arena *arena = Arena_Get(&intern->arena);
+
+  if (upb_FieldDef_IsMap(f)) {
+    upb_MutableMessageValue msgval = upb_Message_Mutable(intern->msg, f, arena);
     MapField_GetPhpWrapper(rv, msgval.map, MapType_Get(f), &intern->arena);
-  } else if (upb_fielddef_isseq(f)) {
-    upb_mutmsgval msgval = upb_msg_mutable(intern->msg, f, arena);
+  } else if (upb_FieldDef_IsRepeated(f)) {
+    upb_MutableMessageValue msgval = upb_Message_Mutable(intern->msg, f, arena);
     RepeatedField_GetPhpWrapper(rv, msgval.array, TypeInfo_Get(f),
                                 &intern->arena);
   } else {
-    upb_msgval msgval = upb_msg_get(intern->msg, f);
+    if (upb_FieldDef_IsSubMessage(f) && !upb_Message_Has(intern->msg, f)) {
+      ZVAL_NULL(rv);
+      return;
+    }
+    upb_MessageValue msgval = upb_Message_Get(intern->msg, f);
     Convert_UpbToPhp(msgval, rv, TypeInfo_Get(f), &intern->arena);
   }
 }
 
-static bool Message_set(Message *intern, const upb_fielddef *f, zval *val) {
-  upb_arena *arena = Arena_Get(&intern->arena);
-  upb_msgval msgval;
+static bool Message_set(Message *intern, const upb_FieldDef *f, zval *val) {
+  upb_Arena *arena = Arena_Get(&intern->arena);
+  upb_MessageValue msgval;
 
-  if (upb_fielddef_ismap(f)) {
+  if (upb_FieldDef_IsMap(f)) {
     msgval.map_val = MapField_GetUpbMap(val, MapType_Get(f), arena);
     if (!msgval.map_val) return false;
-  } else if (upb_fielddef_isseq(f)) {
+  } else if (upb_FieldDef_IsRepeated(f)) {
     msgval.array_val = RepeatedField_GetUpbArray(val, TypeInfo_Get(f), arena);
     if (!msgval.array_val) return false;
+  } else if (upb_FieldDef_IsSubMessage(f) && Z_TYPE_P(val) == IS_NULL) {
+    upb_Message_ClearField(intern->msg, f);
+    return true;
   } else {
     if (!Convert_PhpToUpb(val, &msgval, TypeInfo_Get(f), arena)) return false;
   }
 
-  upb_msg_set(intern->msg, f, msgval, arena);
+  upb_Message_Set(intern->msg, f, msgval, arena);
   return true;
 }
 
-static bool MessageEq(const upb_msg *m1, const upb_msg *m2, const upb_msgdef *m);
+static bool MessageEq(const upb_Message *m1, const upb_Message *m2, const upb_MessageDef *m);
 
 /**
  * ValueEq()
  */
-bool ValueEq(upb_msgval val1, upb_msgval val2, TypeInfo type) {
+bool ValueEq(upb_MessageValue val1, upb_MessageValue val2, TypeInfo type) {
   switch (type.type) {
-    case UPB_TYPE_BOOL:
+    case kUpb_CType_Bool:
       return val1.bool_val == val2.bool_val;
-    case UPB_TYPE_INT32:
-    case UPB_TYPE_UINT32:
-    case UPB_TYPE_ENUM:
+    case kUpb_CType_Int32:
+    case kUpb_CType_UInt32:
+    case kUpb_CType_Enum:
       return val1.int32_val == val2.int32_val;
-    case UPB_TYPE_INT64:
-    case UPB_TYPE_UINT64:
+    case kUpb_CType_Int64:
+    case kUpb_CType_UInt64:
       return val1.int64_val == val2.int64_val;
-    case UPB_TYPE_FLOAT:
+    case kUpb_CType_Float:
       return val1.float_val == val2.float_val;
-    case UPB_TYPE_DOUBLE:
+    case kUpb_CType_Double:
       return val1.double_val == val2.double_val;
-    case UPB_TYPE_STRING:
-    case UPB_TYPE_BYTES:
+    case kUpb_CType_String:
+    case kUpb_CType_Bytes:
       return val1.str_val.size == val2.str_val.size &&
           memcmp(val1.str_val.data, val2.str_val.data, val1.str_val.size) == 0;
-    case UPB_TYPE_MESSAGE:
+    case kUpb_CType_Message:
       return MessageEq(val1.msg_val, val2.msg_val, type.desc->msgdef);
     default:
       return false;
@@ -191,26 +217,25 @@ bool ValueEq(upb_msgval val1, upb_msgval val2, TypeInfo type) {
 /**
  * MessageEq()
  */
-static bool MessageEq(const upb_msg *m1, const upb_msg *m2, const upb_msgdef *m) {
-  upb_msg_field_iter i;
+static bool MessageEq(const upb_Message *m1, const upb_Message *m2, const upb_MessageDef *m) {
+  int n = upb_MessageDef_FieldCount(m);
 
-  for(upb_msg_field_begin(&i, m);
-      !upb_msg_field_done(&i);
-      upb_msg_field_next(&i)) {
-    const upb_fielddef *f = upb_msg_iter_field(&i);
-    upb_msgval val1 = upb_msg_get(m1, f);
-    upb_msgval val2 = upb_msg_get(m2, f);
+  for(int i = 0; i < n; i++) {
+    const upb_FieldDef *f = upb_MessageDef_Field(m, i);
 
-    if (upb_fielddef_haspresence(f)) {
-      if (upb_msg_has(m1, f) != upb_msg_has(m2, f)) {
+    if (upb_FieldDef_HasPresence(f)) {
+      if (upb_Message_Has(m1, f) != upb_Message_Has(m2, f)) {
         return false;
       }
-      if (!upb_msg_has(m1, f)) continue;
+      if (!upb_Message_Has(m1, f)) continue;
     }
 
-    if (upb_fielddef_ismap(f)) {
+    upb_MessageValue val1 = upb_Message_Get(m1, f);
+    upb_MessageValue val2 = upb_Message_Get(m2, f);
+
+    if (upb_FieldDef_IsMap(f)) {
       if (!MapEq(val1.map_val, val2.map_val, MapType_Get(f))) return false;
-    } else if (upb_fielddef_isseq(f)) {
+    } else if (upb_FieldDef_IsRepeated(f)) {
       if (!ArrayEq(val1.array_val, val2.array_val, TypeInfo_Get(f))) return false;
     } else {
       if (!ValueEq(val1, val2, TypeInfo_Get(f))) return false;
@@ -231,7 +256,7 @@ static bool MessageEq(const upb_msg *m1, const upb_msg *m2, const upb_msgdef *m)
 static int Message_compare_objects(zval *m1, zval *m2) {
   Message* intern1 = (Message*)Z_OBJ_P(m1);
   Message* intern2 = (Message*)Z_OBJ_P(m2);
-  const upb_msgdef *m = intern1->desc->msgdef;
+  const upb_MessageDef *m = intern1->desc->msgdef;
 
   if (intern2->desc->msgdef != m) return 1;
 
@@ -260,26 +285,25 @@ static int Message_has_property(PROTO_VAL *obj, PROTO_STR *member,
                                 int has_set_exists,
                                 void **cache_slot) {
   Message* intern = PROTO_VAL_P(obj);
-  const upb_fielddef *f = get_field(intern, member);
+  const upb_FieldDef *f = get_field(intern, member);
 
   if (!f) return 0;
 
-  if (!upb_fielddef_haspresence(f)) {
+  if (!upb_FieldDef_HasPresence(f)) {
     zend_throw_exception_ex(
         NULL, 0,
         "Cannot call isset() on field %s which does not have presence.",
-        ZSTR_VAL(intern->desc->class_entry->name));
+        upb_FieldDef_Name(f));
     return 0;
   }
 
-  return upb_msg_has(intern->msg, f);
+  return upb_Message_Has(intern->msg, f);
 }
 
 /**
  * Message_unset_property()
  *
  * Object handler for unsetting a property. Called when PHP code calls:
- * does any of:
  *
  *   unset($message->foobar);
  *
@@ -295,19 +319,19 @@ static int Message_has_property(PROTO_VAL *obj, PROTO_STR *member,
 static void Message_unset_property(PROTO_VAL *obj, PROTO_STR *member,
                                    void **cache_slot) {
   Message* intern = PROTO_VAL_P(obj);
-  const upb_fielddef *f = get_field(intern, member);
+  const upb_FieldDef *f = get_field(intern, member);
 
   if (!f) return;
 
-  if (!upb_fielddef_haspresence(f)) {
+  if (!upb_FieldDef_HasPresence(f)) {
     zend_throw_exception_ex(
         NULL, 0,
         "Cannot call unset() on field %s which does not have presence.",
-        ZSTR_VAL(intern->desc->class_entry->name));
+        upb_FieldDef_Name(f));
     return;
   }
 
-  upb_msg_clearfield(intern->msg, f);
+  upb_Message_ClearField(intern->msg, f);
 }
 
 
@@ -332,7 +356,7 @@ static void Message_unset_property(PROTO_VAL *obj, PROTO_STR *member,
 static zval *Message_read_property(PROTO_VAL *obj, PROTO_STR *member,
                                    int type, void **cache_slot, zval *rv) {
   Message* intern = PROTO_VAL_P(obj);
-  const upb_fielddef *f = get_field(intern, member);
+  const upb_FieldDef *f = get_field(intern, member);
 
   if (!f) return &EG(uninitialized_zval);
   Message_get(intern, f, rv);
@@ -363,7 +387,7 @@ static zval *Message_read_property(PROTO_VAL *obj, PROTO_STR *member,
 static PROTO_RETURN_VAL Message_write_property(
     PROTO_VAL *obj, PROTO_STR *member, zval *val, void **cache_slot) {
   Message* intern = PROTO_VAL_P(obj);
-  const upb_fielddef *f = get_field(intern, member);
+  const upb_FieldDef *f = get_field(intern, member);
 
   if (f && Message_set(intern, f, val)) {
 #if PHP_VERSION_ID < 70400
@@ -402,11 +426,11 @@ static zval *Message_get_property_ptr_ptr(PROTO_VAL *object, PROTO_STR *member,
  */
 static zend_object *Message_clone_obj(PROTO_VAL *object) {
   Message* intern = PROTO_VAL_P(object);
-  upb_msg *clone = upb_msg_new(intern->desc->msgdef, Arena_Get(&intern->arena));
+  upb_Message *clone = upb_Message_New(intern->desc->msgdef, Arena_Get(&intern->arena));
 
   // TODO: copy unknown fields?
   // TODO: use official upb msg copy function
-  memcpy(clone, intern->msg, upb_msgdef_layout(intern->desc->msgdef)->size);
+  memcpy(clone, intern->msg, upb_MessageDef_MiniTable(intern->desc->msgdef)->size);
   zval ret;
   Message_GetPhpWrapper(&ret, intern->desc, clone, &intern->arena);
   return Z_OBJ_P(&ret);
@@ -426,7 +450,7 @@ static HashTable *Message_get_properties(PROTO_VAL *object) {
 
 // These are documented in the header file.
 
-void Message_GetPhpWrapper(zval *val, const Descriptor *desc, upb_msg *msg,
+void Message_GetPhpWrapper(zval *val, const Descriptor *desc, upb_Message *msg,
                            zval *arena) {
   if (!msg) {
     ZVAL_NULL(val);
@@ -446,27 +470,23 @@ void Message_GetPhpWrapper(zval *val, const Descriptor *desc, upb_msg *msg,
   }
 }
 
-bool Message_GetUpbMessage(zval *val, const Descriptor *desc, upb_arena *arena,
-                           upb_msg **msg) {
+bool Message_GetUpbMessage(zval *val, const Descriptor *desc, upb_Arena *arena,
+                           upb_Message **msg) {
   PBPHP_ASSERT(desc);
 
   if (Z_ISREF_P(val)) {
     ZVAL_DEREF(val);
   }
 
-  if (Z_TYPE_P(val) == IS_NULL) {
-    *msg = NULL;
-    return true;
-  }
-
   if (Z_TYPE_P(val) == IS_OBJECT &&
       instanceof_function(Z_OBJCE_P(val), desc->class_entry)) {
     Message *intern = (Message*)Z_OBJ_P(val);
-    upb_arena_fuse(arena, Arena_Get(&intern->arena));
+    upb_Arena_Fuse(arena, Arena_Get(&intern->arena));
     *msg = intern->msg;
     return true;
   } else {
-    zend_throw_exception_ex(NULL, 0, "Given value is not an instance of %s.",
+    zend_throw_exception_ex(zend_ce_type_error, 0,
+                            "Given value is not an instance of %s.",
                             ZSTR_VAL(desc->class_entry->name));
     return false;
   }
@@ -498,8 +518,8 @@ bool Message_GetUpbMessage(zval *val, const Descriptor *desc, upb_arena *arena,
  *
  * The initializer must be an array.
  */
-bool Message_InitFromPhp(upb_msg *msg, const upb_msgdef *m, zval *init,
-                         upb_arena *arena) {
+bool Message_InitFromPhp(upb_Message *msg, const upb_MessageDef *m, zval *init,
+                         upb_Arena *arena) {
   HashTable* table = HASH_OF(init);
   HashPosition pos;
 
@@ -510,7 +530,7 @@ bool Message_InitFromPhp(upb_msg *msg, const upb_msgdef *m, zval *init,
   if (Z_TYPE_P(init) != IS_ARRAY) {
     zend_throw_exception_ex(NULL, 0,
                             "Initializer for a message %s must be an array.",
-                            upb_msgdef_fullname(m));
+                            upb_MessageDef_FullName(m));
     return false;
   }
 
@@ -519,8 +539,8 @@ bool Message_InitFromPhp(upb_msg *msg, const upb_msgdef *m, zval *init,
   while (true) {  // Iterate over key/value pairs.
     zval key;
     zval *val;
-    const upb_fielddef *f;
-    upb_msgval msgval;
+    const upb_FieldDef *f;
+    upb_MessageValue msgval;
 
     zend_hash_get_current_key_zval_ex(table, &key, &pos);
     val = zend_hash_get_current_data_ex(table, &pos);
@@ -531,7 +551,7 @@ bool Message_InitFromPhp(upb_msg *msg, const upb_msgdef *m, zval *init,
       ZVAL_DEREF(val);
     }
 
-    f = upb_msgdef_ntof(m, Z_STRVAL_P(&key), Z_STRLEN_P(&key));
+    f = upb_MessageDef_FindFieldByNameWithSize(m, Z_STRVAL_P(&key), Z_STRLEN_P(&key));
 
     if (!f) {
       zend_throw_exception_ex(NULL, 0,
@@ -539,10 +559,10 @@ bool Message_InitFromPhp(upb_msg *msg, const upb_msgdef *m, zval *init,
       return false;
     }
 
-    if (upb_fielddef_ismap(f)) {
+    if (upb_FieldDef_IsMap(f)) {
       msgval.map_val = MapField_GetUpbMap(val, MapType_Get(f), arena);
       if (!msgval.map_val) return false;
-    } else if (upb_fielddef_isseq(f)) {
+    } else if (upb_FieldDef_IsRepeated(f)) {
       msgval.array_val = RepeatedField_GetUpbArray(val, TypeInfo_Get(f), arena);
       if (!msgval.array_val) return false;
     } else {
@@ -551,7 +571,7 @@ bool Message_InitFromPhp(upb_msg *msg, const upb_msgdef *m, zval *init,
       }
     }
 
-    upb_msg_set(msg, f, msgval, arena);
+    upb_Message_Set(msg, f, msgval, arena);
     zend_hash_move_forward_ex(table, &pos);
     zval_dtor(&key);
   }
@@ -559,7 +579,7 @@ bool Message_InitFromPhp(upb_msg *msg, const upb_msgdef *m, zval *init,
 
 static void Message_Initialize(Message *intern, const Descriptor *desc) {
   intern->desc = desc;
-  intern->msg = upb_msg_new(desc->msgdef, Arena_Get(&intern->arena));
+  intern->msg = upb_Message_New(desc->msgdef, Arena_Get(&intern->arena));
   ObjCache_Add(intern->msg, &intern->std);
 }
 
@@ -573,7 +593,7 @@ PHP_METHOD(Message, __construct) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
   const Descriptor* desc;
   zend_class_entry *ce = Z_OBJCE_P(getThis());
-  upb_arena *arena = Arena_Get(&intern->arena);
+  upb_Arena *arena = Arena_Get(&intern->arena);
   zval *init_arr = NULL;
 
   // This descriptor should always be available, as the generated __construct
@@ -583,10 +603,9 @@ PHP_METHOD(Message, __construct) {
   // will trigger an infinite construction loop and blow the stack.  We
   // temporarily clear create_object to break this loop (see check in
   // NameMap_GetMessage()).
-  PBPHP_ASSERT(ce->create_object == Message_create);
-  ce->create_object = NULL;
+  NameMap_EnterConstructor(ce);
   desc = Descriptor_GetFromClassEntry(ce);
-  ce->create_object = Message_create;
+  NameMap_ExitConstructor(ce);
 
   if (!desc) {
     zend_throw_exception_ex(
@@ -596,7 +615,6 @@ PHP_METHOD(Message, __construct) {
     return;
   }
 
-
   Message_Initialize(intern, desc);
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &init_arr) == FAILURE) {
@@ -615,7 +633,7 @@ PHP_METHOD(Message, __construct) {
  */
 PHP_METHOD(Message, discardUnknownFields) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  upb_msg_discardunknown(intern->msg, intern->desc->msgdef, 64);
+  upb_Message_DiscardUnknown(intern->msg, intern->desc->msgdef, 64);
 }
 
 /**
@@ -625,7 +643,7 @@ PHP_METHOD(Message, discardUnknownFields) {
  */
 PHP_METHOD(Message, clear) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  upb_msg_clear(intern->msg, intern->desc->msgdef);
+  upb_Message_Clear(intern->msg, intern->desc->msgdef);
 }
 
 /**
@@ -637,8 +655,8 @@ PHP_METHOD(Message, clear) {
 PHP_METHOD(Message, mergeFrom) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
   Message* from;
-  upb_arena *arena = Arena_Get(&intern->arena);
-  const upb_msglayout *l = upb_msgdef_layout(intern->desc->msgdef);
+  upb_Arena *arena = Arena_Get(&intern->arena);
+  const upb_MiniTable *l = upb_MessageDef_MiniTable(intern->desc->msgdef);
   zval* value;
   char *pb;
   size_t size;
@@ -657,14 +675,15 @@ PHP_METHOD(Message, mergeFrom) {
 
   // TODO(haberman): use a temp arena for this once we can make upb_decode()
   // copy strings.
-  pb = upb_encode(from->msg, l, arena, &size);
+  pb = upb_Encode(from->msg, l, 0, arena, &size);
 
   if (!pb) {
     zend_throw_exception_ex(NULL, 0, "Max nesting exceeded");
     return;
   }
 
-  ok = upb_decode(pb, size, intern->msg, l, arena);
+  ok = upb_Decode(pb, size, intern->msg, l, NULL, 0, arena) ==
+       kUpb_DecodeStatus_Ok;
   PBPHP_ASSERT(ok);
 }
 
@@ -679,8 +698,8 @@ PHP_METHOD(Message, mergeFromString) {
   char *data = NULL;
   char *data_copy = NULL;
   zend_long data_len;
-  const upb_msglayout *l = upb_msgdef_layout(intern->desc->msgdef);
-  upb_arena *arena = Arena_Get(&intern->arena);
+  const upb_MiniTable *l = upb_MessageDef_MiniTable(intern->desc->msgdef);
+  upb_Arena *arena = Arena_Get(&intern->arena);
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &data, &data_len) ==
       FAILURE) {
@@ -688,10 +707,11 @@ PHP_METHOD(Message, mergeFromString) {
   }
 
   // TODO(haberman): avoid this copy when we can make the decoder copy.
-  data_copy = upb_arena_malloc(arena, data_len);
+  data_copy = upb_Arena_Malloc(arena, data_len);
   memcpy(data_copy, data, data_len);
 
-  if (!upb_decode(data_copy, data_len, intern->msg, l, arena)) {
+  if (upb_Decode(data_copy, data_len, intern->msg, l, NULL, 0, arena) !=
+      kUpb_DecodeStatus_Ok) {
     zend_throw_exception_ex(NULL, 0, "Error occurred during parsing");
     return;
   }
@@ -705,21 +725,21 @@ PHP_METHOD(Message, mergeFromString) {
  */
 PHP_METHOD(Message, serializeToString) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  const upb_msglayout *l = upb_msgdef_layout(intern->desc->msgdef);
-  upb_arena *tmp_arena = upb_arena_new();
+  const upb_MiniTable *l = upb_MessageDef_MiniTable(intern->desc->msgdef);
+  upb_Arena *tmp_arena = upb_Arena_New();
   char *data;
   size_t size;
 
-  data = upb_encode(intern->msg, l, tmp_arena, &size);
+  data = upb_Encode(intern->msg, l, 0, tmp_arena, &size);
 
   if (!data) {
     zend_throw_exception_ex(NULL, 0, "Error occurred during serialization");
-    upb_arena_free(tmp_arena);
+    upb_Arena_Free(tmp_arena);
     return;
   }
 
   RETVAL_STRINGL(data, size);
-  upb_arena_free(tmp_arena);
+  upb_Arena_Free(tmp_arena);
 }
 
 /**
@@ -733,8 +753,8 @@ PHP_METHOD(Message, mergeFromJsonString) {
   char *data = NULL;
   char *data_copy = NULL;
   zend_long data_len;
-  upb_arena *arena = Arena_Get(&intern->arena);
-  upb_status status;
+  upb_Arena *arena = Arena_Get(&intern->arena);
+  upb_Status status;
   zend_bool ignore_json_unknown = false;
   int options = 0;
 
@@ -744,20 +764,20 @@ PHP_METHOD(Message, mergeFromJsonString) {
   }
 
   // TODO(haberman): avoid this copy when we can make the decoder copy.
-  data_copy = upb_arena_malloc(arena, data_len + 1);
+  data_copy = upb_Arena_Malloc(arena, data_len + 1);
   memcpy(data_copy, data, data_len);
   data_copy[data_len] = '\0';
 
   if (ignore_json_unknown) {
-    options |= UPB_JSONDEC_IGNOREUNKNOWN;
+    options |= upb_JsonDecode_IgnoreUnknown;
   }
 
-  upb_status_clear(&status);
-  if (!upb_json_decode(data_copy, data_len, intern->msg, intern->desc->msgdef,
+  upb_Status_Clear(&status);
+  if (!upb_JsonDecode(data_copy, data_len, intern->msg, intern->desc->msgdef,
                        DescriptorPool_GetSymbolTable(), options, arena,
                        &status)) {
     zend_throw_exception_ex(NULL, 0, "Error occurred during parsing: %s",
-                            upb_status_errmsg(&status));
+                            upb_Status_ErrorMessage(&status));
     return;
   }
 }
@@ -774,7 +794,7 @@ PHP_METHOD(Message, serializeToJsonString) {
   int options = 0;
   char buf[1024];
   zend_bool preserve_proto_fieldnames = false;
-  upb_status status;
+  upb_Status status;
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b",
                             &preserve_proto_fieldnames) == FAILURE) {
@@ -782,24 +802,24 @@ PHP_METHOD(Message, serializeToJsonString) {
   }
 
   if (preserve_proto_fieldnames) {
-    options |= UPB_JSONENC_PROTONAMES;
+    options |= upb_JsonEncode_UseProtoNames;
   }
 
-  upb_status_clear(&status);
-  size = upb_json_encode(intern->msg, intern->desc->msgdef,
+  upb_Status_Clear(&status);
+  size = upb_JsonEncode(intern->msg, intern->desc->msgdef,
                          DescriptorPool_GetSymbolTable(), options, buf,
                          sizeof(buf), &status);
 
-  if (!upb_ok(&status)) {
+  if (!upb_Status_IsOk(&status)) {
     zend_throw_exception_ex(NULL, 0,
                             "Error occurred during JSON serialization: %s",
-                            upb_status_errmsg(&status));
+                            upb_Status_ErrorMessage(&status));
     return;
   }
 
   if (size >= sizeof(buf)) {
     char *buf2 = malloc(size + 1);
-    upb_json_encode(intern->msg, intern->desc->msgdef,
+    upb_JsonEncode(intern->msg, intern->desc->msgdef,
                     DescriptorPool_GetSymbolTable(), options, buf2, size + 1,
                     &status);
     RETVAL_STRINGL(buf2, size);
@@ -825,29 +845,29 @@ PHP_METHOD(Message, serializeToJsonString) {
 PHP_METHOD(Message, readWrapperValue) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
   char* member;
-  const upb_fielddef *f;
+  const upb_FieldDef *f;
   zend_long size;
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &member, &size) == FAILURE) {
     return;
   }
 
-  f = upb_msgdef_ntof(intern->desc->msgdef, member, size);
+  f = upb_MessageDef_FindFieldByNameWithSize(intern->desc->msgdef, member, size);
 
-  if (!f || !upb_msgdef_iswrapper(upb_fielddef_msgsubdef(f))) {
+  if (!f || !IsWrapper(upb_FieldDef_MessageSubDef(f))) {
     zend_throw_exception_ex(NULL, 0, "Message %s has no field %s",
-                            upb_msgdef_fullname(intern->desc->msgdef), member);
+                            upb_MessageDef_FullName(intern->desc->msgdef), member);
     return;
   }
 
-  if (upb_msg_has(intern->msg, f)) {
-    const upb_msg *wrapper = upb_msg_get(intern->msg, f).msg_val;
-    const upb_msgdef *m = upb_fielddef_msgsubdef(f);
-    const upb_fielddef *val_f = upb_msgdef_itof(m, 1);
-    upb_msgval msgval = upb_msg_get(wrapper, val_f);
+  if (upb_Message_Has(intern->msg, f)) {
+    const upb_Message *wrapper = upb_Message_Get(intern->msg, f).msg_val;
+    const upb_MessageDef *m = upb_FieldDef_MessageSubDef(f);
+    const upb_FieldDef *val_f = upb_MessageDef_FindFieldByNumber(m, 1);
+    upb_MessageValue msgval = upb_Message_Get(wrapper, val_f);
     zval ret;
     Convert_UpbToPhp(msgval, &ret, TypeInfo_Get(val_f), &intern->arena);
-    RETURN_ZVAL(&ret, 1, 0);
+    RETURN_COPY_VALUE(&ret);
   } else {
     RETURN_NULL();
   }
@@ -870,10 +890,10 @@ PHP_METHOD(Message, readWrapperValue) {
  */
 PHP_METHOD(Message, writeWrapperValue) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  upb_arena *arena = Arena_Get(&intern->arena);
+  upb_Arena *arena = Arena_Get(&intern->arena);
   char* member;
-  const upb_fielddef *f;
-  upb_msgval msgval;
+  const upb_FieldDef *f;
+  upb_MessageValue msgval;
   zend_long size;
   zval* val;
 
@@ -882,11 +902,11 @@ PHP_METHOD(Message, writeWrapperValue) {
     return;
   }
 
-  f = upb_msgdef_ntof(intern->desc->msgdef, member, size);
+  f = upb_MessageDef_FindFieldByNameWithSize(intern->desc->msgdef, member, size);
 
-  if (!f || !upb_msgdef_iswrapper(upb_fielddef_msgsubdef(f))) {
+  if (!f || !IsWrapper(upb_FieldDef_MessageSubDef(f))) {
     zend_throw_exception_ex(NULL, 0, "Message %s has no field %s",
-                            upb_msgdef_fullname(intern->desc->msgdef), member);
+                            upb_MessageDef_FullName(intern->desc->msgdef), member);
     return;
   }
 
@@ -895,18 +915,18 @@ PHP_METHOD(Message, writeWrapperValue) {
   }
 
   if (Z_TYPE_P(val) == IS_NULL) {
-    upb_msg_clearfield(intern->msg, f);
+    upb_Message_ClearField(intern->msg, f);
   } else {
-    const upb_msgdef *m = upb_fielddef_msgsubdef(f);
-    const upb_fielddef *val_f = upb_msgdef_itof(m, 1);
-    upb_msg *wrapper;
+    const upb_MessageDef *m = upb_FieldDef_MessageSubDef(f);
+    const upb_FieldDef *val_f = upb_MessageDef_FindFieldByNumber(m, 1);
+    upb_Message *wrapper;
 
     if (!Convert_PhpToUpb(val, &msgval, TypeInfo_Get(val_f), arena)) {
       return;  // Error is already set.
     }
 
-    wrapper = upb_msg_mutable(intern->msg, f, arena).msg;
-    upb_msg_set(wrapper, val_f, msgval, arena);
+    wrapper = upb_Message_Mutable(intern->msg, f, arena).msg;
+    upb_Message_Set(wrapper, val_f, msgval, arena);
   }
 }
 
@@ -920,8 +940,8 @@ PHP_METHOD(Message, writeWrapperValue) {
  */
 PHP_METHOD(Message, whichOneof) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  const upb_oneofdef* oneof;
-  const upb_fielddef* field;
+  const upb_OneofDef* oneof;
+  const upb_FieldDef* field;
   char* name;
   zend_long len;
 
@@ -929,16 +949,16 @@ PHP_METHOD(Message, whichOneof) {
     return;
   }
 
-  oneof = upb_msgdef_ntoo(intern->desc->msgdef, name, len);
+  oneof = upb_MessageDef_FindOneofByNameWithSize(intern->desc->msgdef, name, len);
 
   if (!oneof) {
     zend_throw_exception_ex(NULL, 0, "Message %s has no oneof %s",
-                            upb_msgdef_fullname(intern->desc->msgdef), name);
+                            upb_MessageDef_FullName(intern->desc->msgdef), name);
     return;
   }
 
-  field = upb_msg_whichoneof(intern->msg, oneof);
-  RETURN_STRING(field ? upb_fielddef_name(field) : "");
+  field = upb_Message_WhichOneof(intern->msg, oneof);
+  RETURN_STRING(field ? upb_FieldDef_Name(field) : "");
 }
 
 /**
@@ -957,21 +977,21 @@ PHP_METHOD(Message, whichOneof) {
 PHP_METHOD(Message, hasOneof) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
   zend_long field_num;
-  const upb_fielddef* f;
+  const upb_FieldDef* f;
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &field_num) == FAILURE) {
     return;
   }
 
-  f = upb_msgdef_itof(intern->desc->msgdef, field_num);
+  f = upb_MessageDef_FindFieldByNumber(intern->desc->msgdef, field_num);
 
-  if (!f || !upb_fielddef_realcontainingoneof(f)) {
+  if (!f || !upb_FieldDef_RealContainingOneof(f)) {
     php_error_docref(NULL, E_USER_ERROR,
                      "Internal error, no such oneof field %d\n",
                      (int)field_num);
   }
 
-  RETVAL_BOOL(upb_msg_has(intern->msg, f));
+  RETVAL_BOOL(upb_Message_Has(intern->msg, f));
 }
 
 /**
@@ -990,31 +1010,31 @@ PHP_METHOD(Message, hasOneof) {
 PHP_METHOD(Message, readOneof) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
   zend_long field_num;
-  const upb_fielddef* f;
+  const upb_FieldDef* f;
   zval ret;
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &field_num) == FAILURE) {
     return;
   }
 
-  f = upb_msgdef_itof(intern->desc->msgdef, field_num);
+  f = upb_MessageDef_FindFieldByNumber(intern->desc->msgdef, field_num);
 
-  if (!f || !upb_fielddef_realcontainingoneof(f)) {
+  if (!f || !upb_FieldDef_RealContainingOneof(f)) {
     php_error_docref(NULL, E_USER_ERROR,
                      "Internal error, no such oneof field %d\n",
                      (int)field_num);
   }
 
-  if (upb_fielddef_issubmsg(f) && !upb_msg_has(intern->msg, f)) {
+  if (upb_FieldDef_IsSubMessage(f) && !upb_Message_Has(intern->msg, f)) {
     RETURN_NULL();
   }
 
   {
-    upb_msgval msgval = upb_msg_get(intern->msg, f);
+    upb_MessageValue msgval = upb_Message_Get(intern->msg, f);
     Convert_UpbToPhp(msgval, &ret, TypeInfo_Get(f), &intern->arena);
   }
 
-  RETURN_ZVAL(&ret, 1, 0);
+  RETURN_COPY_VALUE(&ret);
 }
 
 /**
@@ -1040,9 +1060,9 @@ PHP_METHOD(Message, readOneof) {
 PHP_METHOD(Message, writeOneof) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
   zend_long field_num;
-  const upb_fielddef* f;
-  upb_arena *arena = Arena_Get(&intern->arena);
-  upb_msgval msgval;
+  const upb_FieldDef* f;
+  upb_Arena *arena = Arena_Get(&intern->arena);
+  upb_MessageValue msgval;
   zval* val;
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "lz", &field_num, &val) ==
@@ -1050,19 +1070,31 @@ PHP_METHOD(Message, writeOneof) {
     return;
   }
 
-  f = upb_msgdef_itof(intern->desc->msgdef, field_num);
+  f = upb_MessageDef_FindFieldByNumber(intern->desc->msgdef, field_num);
 
-  if (!Convert_PhpToUpb(val, &msgval, TypeInfo_Get(f), arena)) {
+  if (upb_FieldDef_IsSubMessage(f) && Z_TYPE_P(val) == IS_NULL) {
+    upb_Message_ClearField(intern->msg, f);
+    return;
+  } else if (!Convert_PhpToUpb(val, &msgval, TypeInfo_Get(f), arena)) {
     return;
   }
 
-  upb_msg_set(intern->msg, f, msgval, arena);
+  upb_Message_Set(intern->msg, f, msgval, arena);
 }
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_construct, 0, 0, 0)
+  ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mergeFrom, 0, 0, 1)
   ZEND_ARG_INFO(0, data)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_mergeFromWithArg, 0, 0, 1)
+  ZEND_ARG_INFO(0, data)
+  ZEND_ARG_INFO(0, arg)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_read, 0, 0, 1)
   ZEND_ARG_INFO(0, field)
 ZEND_END_ARG_INFO()
@@ -1078,7 +1110,7 @@ static zend_function_entry Message_methods[] = {
   PHP_ME(Message, serializeToString,     arginfo_void,      ZEND_ACC_PUBLIC)
   PHP_ME(Message, mergeFromString,       arginfo_mergeFrom, ZEND_ACC_PUBLIC)
   PHP_ME(Message, serializeToJsonString, arginfo_void,      ZEND_ACC_PUBLIC)
-  PHP_ME(Message, mergeFromJsonString,   arginfo_mergeFrom, ZEND_ACC_PUBLIC)
+  PHP_ME(Message, mergeFromJsonString,   arginfo_mergeFromWithArg, ZEND_ACC_PUBLIC)
   PHP_ME(Message, mergeFrom,             arginfo_mergeFrom, ZEND_ACC_PUBLIC)
   PHP_ME(Message, readWrapperValue,      arginfo_read,      ZEND_ACC_PROTECTED)
   PHP_ME(Message, writeWrapperValue,     arginfo_write,     ZEND_ACC_PROTECTED)
@@ -1086,7 +1118,7 @@ static zend_function_entry Message_methods[] = {
   PHP_ME(Message, readOneof,             arginfo_read,      ZEND_ACC_PROTECTED)
   PHP_ME(Message, writeOneof,            arginfo_write,     ZEND_ACC_PROTECTED)
   PHP_ME(Message, whichOneof,            arginfo_read,      ZEND_ACC_PROTECTED)
-  PHP_ME(Message, __construct,           arginfo_void,      ZEND_ACC_PROTECTED)
+  PHP_ME(Message, __construct,           arginfo_construct, ZEND_ACC_PROTECTED)
   ZEND_FE_END
 };
 
@@ -1094,26 +1126,27 @@ static zend_function_entry Message_methods[] = {
 
 static const char TYPE_URL_PREFIX[] = "type.googleapis.com/";
 
-static upb_msgval Message_getval(Message *intern, const char *field_name) {
-  const upb_fielddef *f = upb_msgdef_ntofz(intern->desc->msgdef, field_name);
+static upb_MessageValue Message_getval(Message *intern, const char *field_name) {
+  const upb_FieldDef *f = upb_MessageDef_FindFieldByName(intern->desc->msgdef, field_name);
   PBPHP_ASSERT(f);
-  return upb_msg_get(intern->msg, f);
+  return upb_Message_Get(intern->msg, f);
 }
 
 static void Message_setval(Message *intern, const char *field_name,
-                           upb_msgval val) {
-  const upb_fielddef *f = upb_msgdef_ntofz(intern->desc->msgdef, field_name);
+                           upb_MessageValue val) {
+  const upb_FieldDef *f =
+      upb_MessageDef_FindFieldByName(intern->desc->msgdef, field_name);
   PBPHP_ASSERT(f);
-  return upb_msg_set(intern->msg, f, val, Arena_Get(&intern->arena));
+  upb_Message_Set(intern->msg, f, val, Arena_Get(&intern->arena));
 }
 
-static upb_msgval StringVal(upb_strview view) {
-  upb_msgval ret;
+static upb_MessageValue StringVal(upb_StringView view) {
+  upb_MessageValue ret;
   ret.str_val = view;
   return ret;
 }
 
-static bool TryStripUrlPrefix(upb_strview *str) {
+static bool TryStripUrlPrefix(upb_StringView *str) {
   size_t size = strlen(TYPE_URL_PREFIX);
   if (str->size < size || memcmp(TYPE_URL_PREFIX, str->data, size) != 0) {
     return false;
@@ -1123,17 +1156,17 @@ static bool TryStripUrlPrefix(upb_strview *str) {
   return true;
 }
 
-static bool StrViewEq(upb_strview view, const char *str) {
+static bool StrViewEq(upb_StringView view, const char *str) {
   size_t size = strlen(str);
   return view.size == size && memcmp(view.data, str, size) == 0;
 }
 
 PHP_METHOD(google_protobuf_Any, unpack) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  upb_strview type_url = Message_getval(intern, "type_url").str_val;
-  upb_strview value = Message_getval(intern, "value").str_val;
-  upb_symtab *symtab = DescriptorPool_GetSymbolTable();
-  const upb_msgdef *m;
+  upb_StringView type_url = Message_getval(intern, "type_url").str_val;
+  upb_StringView value = Message_getval(intern, "value").str_val;
+  upb_DefPool *symtab = DescriptorPool_GetSymbolTable();
+  const upb_MessageDef *m;
   Descriptor *desc;
   zval ret;
 
@@ -1145,7 +1178,7 @@ PHP_METHOD(google_protobuf_Any, unpack) {
     return;
   }
 
-  m = upb_symtab_lookupmsg2(symtab, type_url.data, type_url.size);
+  m = upb_DefPool_FindMessageByNameWithSize(symtab, type_url.data, type_url.size);
 
   if (m == NULL) {
     zend_throw_exception(
@@ -1162,25 +1195,27 @@ PHP_METHOD(google_protobuf_Any, unpack) {
   ZVAL_OBJ(&ret, obj);
 
   // Get value.
-  if (!upb_decode(value.data, value.size, msg->msg,
-                  upb_msgdef_layout(desc->msgdef), Arena_Get(&msg->arena))) {
+  if (upb_Decode(value.data, value.size, msg->msg,
+                 upb_MessageDef_MiniTable(desc->msgdef), NULL, 0,
+                 Arena_Get(&msg->arena)) != kUpb_DecodeStatus_Ok) {
     zend_throw_exception_ex(NULL, 0, "Error occurred during parsing");
+    zval_dtor(&ret);
     return;
   }
 
   // Fuse since the parsed message could alias "value".
-  upb_arena_fuse(Arena_Get(&intern->arena), Arena_Get(&msg->arena));
+  upb_Arena_Fuse(Arena_Get(&intern->arena), Arena_Get(&msg->arena));
 
-  RETURN_ZVAL(&ret, 1, 0);
+  RETURN_COPY_VALUE(&ret);
 }
 
 PHP_METHOD(google_protobuf_Any, pack) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  upb_arena *arena = Arena_Get(&intern->arena);
+  upb_Arena *arena = Arena_Get(&intern->arena);
   zval *val;
   Message *msg;
-  upb_strview value;
-  upb_strview type_url;
+  upb_StringView value;
+  upb_StringView type_url;
   const char *full_name;
   char *buf;
 
@@ -1197,14 +1232,14 @@ PHP_METHOD(google_protobuf_Any, pack) {
   msg = (Message*)Z_OBJ_P(val);
 
   // Serialize and set value.
-  value.data = upb_encode(msg->msg, upb_msgdef_layout(msg->desc->msgdef), arena,
-                          &value.size);
+  value.data = upb_Encode(msg->msg, upb_MessageDef_MiniTable(msg->desc->msgdef),
+                          0, arena, &value.size);
   Message_setval(intern, "value", StringVal(value));
 
   // Set type url: type_url_prefix + fully_qualified_name
-  full_name = upb_msgdef_fullname(msg->desc->msgdef);
+  full_name = upb_MessageDef_FullName(msg->desc->msgdef);
   type_url.size = strlen(TYPE_URL_PREFIX) + strlen(full_name);
-  buf = upb_arena_malloc(arena, type_url.size + 1);
+  buf = upb_Arena_Malloc(arena, type_url.size + 1);
   memcpy(buf, TYPE_URL_PREFIX, strlen(TYPE_URL_PREFIX));
   memcpy(buf + strlen(TYPE_URL_PREFIX), full_name, strlen(full_name));
   type_url.data = buf;
@@ -1213,9 +1248,9 @@ PHP_METHOD(google_protobuf_Any, pack) {
 
 PHP_METHOD(google_protobuf_Any, is) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  upb_strview type_url = Message_getval(intern, "type_url").str_val;
+  upb_StringView type_url = Message_getval(intern, "type_url").str_val;
   zend_class_entry *klass = NULL;
-  const upb_msgdef *m;
+  const upb_MessageDef *m;
 
   if (zend_parse_parameters(ZEND_NUM_ARGS(), "C", &klass) ==
       FAILURE) {
@@ -1229,7 +1264,7 @@ PHP_METHOD(google_protobuf_Any, is) {
   }
 
   RETURN_BOOL(TryStripUrlPrefix(&type_url) &&
-              StrViewEq(type_url, upb_msgdef_fullname(m)));
+              StrViewEq(type_url, upb_MessageDef_FullName(m)));
 }
 
 PHP_METHOD(google_protobuf_Timestamp, fromDateTime) {
@@ -1238,6 +1273,7 @@ PHP_METHOD(google_protobuf_Timestamp, fromDateTime) {
   const char *classname = "\\DatetimeInterface";
   zend_string *classname_str = zend_string_init(classname, strlen(classname), 0);
   zend_class_entry *date_interface_ce = zend_lookup_class(classname_str);
+  zend_string_release(classname_str);
 
   if (date_interface_ce == NULL) {
     zend_error(E_ERROR, "Make sure date extension is enabled.");
@@ -1250,7 +1286,7 @@ PHP_METHOD(google_protobuf_Timestamp, fromDateTime) {
     return;
   }
 
-  upb_msgval timestamp_seconds;
+  upb_MessageValue timestamp_seconds;
   {
     zval retval;
     zval function_name;
@@ -1260,7 +1296,7 @@ PHP_METHOD(google_protobuf_Timestamp, fromDateTime) {
     if (call_user_function(EG(function_table), NULL, &function_name, &retval, 1,
                            datetime) == FAILURE ||
         !Convert_PhpToUpb(&retval, &timestamp_seconds,
-                          TypeInfo_FromType(UPB_TYPE_INT64), NULL)) {
+                          TypeInfo_FromType(kUpb_CType_Int64), NULL)) {
       zend_error(E_ERROR, "Cannot get timestamp from DateTime.");
       return;
     }
@@ -1269,7 +1305,7 @@ PHP_METHOD(google_protobuf_Timestamp, fromDateTime) {
     zval_dtor(&function_name);
   }
 
-  upb_msgval timestamp_nanos;
+  upb_MessageValue timestamp_nanos;
   {
     zval retval;
     zval function_name;
@@ -1286,7 +1322,7 @@ PHP_METHOD(google_protobuf_Timestamp, fromDateTime) {
     if (call_user_function(EG(function_table), NULL, &function_name, &retval, 2,
                            params) == FAILURE ||
         !Convert_PhpToUpb(&retval, &timestamp_nanos,
-                          TypeInfo_FromType(UPB_TYPE_INT32), NULL)) {
+                          TypeInfo_FromType(kUpb_CType_Int32), NULL)) {
       zend_error(E_ERROR, "Cannot format DateTime.");
       return;
     }
@@ -1306,8 +1342,8 @@ PHP_METHOD(google_protobuf_Timestamp, fromDateTime) {
 
 PHP_METHOD(google_protobuf_Timestamp, toDateTime) {
   Message* intern = (Message*)Z_OBJ_P(getThis());
-  upb_msgval seconds = Message_getval(intern, "seconds");
-  upb_msgval nanos = Message_getval(intern, "nanos");
+  upb_MessageValue seconds = Message_getval(intern, "seconds");
+  upb_MessageValue nanos = Message_getval(intern, "nanos");
 
   // Get formatted time string.
   char formatted_time[32];
