@@ -32,14 +32,16 @@
 // Author: tibell@google.com (Johan Tibell)
 
 #include <google/protobuf/pyext/extension_dict.h>
+
+#include <cstdint>
 #include <memory>
 
 #include <google/protobuf/stubs/logging.h>
 #include <google/protobuf/stubs/common.h>
+#include <google/protobuf/descriptor.pb.h>
 #include <google/protobuf/descriptor.h>
 #include <google/protobuf/dynamic_message.h>
 #include <google/protobuf/message.h>
-#include <google/protobuf/descriptor.pb.h>
 #include <google/protobuf/pyext/descriptor.h>
 #include <google/protobuf/pyext/message.h>
 #include <google/protobuf/pyext/message_factory.h>
@@ -47,17 +49,13 @@
 #include <google/protobuf/pyext/repeated_scalar_container.h>
 #include <google/protobuf/pyext/scoped_pyobject_ptr.h>
 
-#if PY_MAJOR_VERSION >= 3
-  #if PY_VERSION_HEX < 0x03030000
-    #error "Python 3.0 - 3.2 are not supported."
-  #endif
-#define PyString_AsStringAndSize(ob, charpp, sizep)                           \
-  (PyUnicode_Check(ob) ? ((*(charpp) = const_cast<char*>(                     \
-                               PyUnicode_AsUTF8AndSize(ob, (sizep)))) == NULL \
-                              ? -1                                            \
-                              : 0)                                            \
-                       : PyBytes_AsStringAndSize(ob, (charpp), (sizep)))
-#endif
+#define PyString_AsStringAndSize(ob, charpp, sizep)              \
+  (PyUnicode_Check(ob)                                           \
+       ? ((*(charpp) = const_cast<char*>(                        \
+               PyUnicode_AsUTF8AndSize(ob, (sizep)))) == nullptr \
+              ? -1                                               \
+              : 0)                                               \
+       : PyBytes_AsStringAndSize(ob, (charpp), (sizep)))
 
 namespace google {
 namespace protobuf {
@@ -133,11 +131,11 @@ static void DeallocExtensionIterator(PyObject* _self) {
 
 PyObject* subscript(ExtensionDict* self, PyObject* key) {
   const FieldDescriptor* descriptor = cmessage::GetExtensionDescriptor(key);
-  if (descriptor == NULL) {
-    return NULL;
+  if (descriptor == nullptr) {
+    return nullptr;
   }
   if (!CheckFieldBelongsToMessage(descriptor, self->parent->message)) {
-    return NULL;
+    return nullptr;
   }
 
   if (descriptor->label() != FieldDescriptor::LABEL_REPEATED &&
@@ -157,8 +155,8 @@ PyObject* subscript(ExtensionDict* self, PyObject* key) {
     // TODO(plabatut): consider building the class on the fly!
     ContainerBase* sub_message = cmessage::InternalGetSubMessage(
         self->parent, descriptor);
-    if (sub_message == NULL) {
-      return NULL;
+    if (sub_message == nullptr) {
+      return nullptr;
     }
     (*self->parent->composite_fields)[descriptor] = sub_message;
     return sub_message->AsPyObject();
@@ -181,39 +179,43 @@ PyObject* subscript(ExtensionDict* self, PyObject* key) {
           descriptor->message_type());
       ScopedPyObjectPtr message_class_handler(
         reinterpret_cast<PyObject*>(message_class));
-      if (message_class == NULL) {
-        return NULL;
+      if (message_class == nullptr) {
+        return nullptr;
       }
       ContainerBase* py_container = repeated_composite_container::NewContainer(
           self->parent, descriptor, message_class);
-      if (py_container == NULL) {
-        return NULL;
+      if (py_container == nullptr) {
+        return nullptr;
       }
       (*self->parent->composite_fields)[descriptor] = py_container;
       return py_container->AsPyObject();
     } else {
       ContainerBase* py_container = repeated_scalar_container::NewContainer(
           self->parent, descriptor);
-      if (py_container == NULL) {
-        return NULL;
+      if (py_container == nullptr) {
+        return nullptr;
       }
       (*self->parent->composite_fields)[descriptor] = py_container;
       return py_container->AsPyObject();
     }
   }
   PyErr_SetString(PyExc_ValueError, "control reached unexpected line");
-  return NULL;
+  return nullptr;
 }
 
 int ass_subscript(ExtensionDict* self, PyObject* key, PyObject* value) {
   const FieldDescriptor* descriptor = cmessage::GetExtensionDescriptor(key);
-  if (descriptor == NULL) {
+  if (descriptor == nullptr) {
     return -1;
   }
   if (!CheckFieldBelongsToMessage(descriptor, self->parent->message)) {
     return -1;
   }
 
+  if (value == nullptr) {
+    return cmessage::ClearFieldByDescriptor(self->parent, descriptor);
+  }
+
   if (descriptor->label() != FieldDescriptor::LABEL_OPTIONAL ||
       descriptor->cpp_type() == FieldDescriptor::CPPTYPE_MESSAGE) {
     PyErr_SetString(PyExc_TypeError, "Extension is repeated and/or composite "
@@ -231,16 +233,16 @@ PyObject* _FindExtensionByName(ExtensionDict* self, PyObject* arg) {
   char* name;
   Py_ssize_t name_size;
   if (PyString_AsStringAndSize(arg, &name, &name_size) < 0) {
-    return NULL;
+    return nullptr;
   }
 
   PyDescriptorPool* pool = cmessage::GetFactoryForMessage(self->parent)->pool;
   const FieldDescriptor* message_extension =
-      pool->pool->FindExtensionByName(string(name, name_size));
-  if (message_extension == NULL) {
+      pool->pool->FindExtensionByName(StringParam(name, name_size));
+  if (message_extension == nullptr) {
     // Is is the name of a message set extension?
-    const Descriptor* message_descriptor = pool->pool->FindMessageTypeByName(
-        string(name, name_size));
+    const Descriptor* message_descriptor =
+        pool->pool->FindMessageTypeByName(StringParam(name, name_size));
     if (message_descriptor && message_descriptor->extension_count() > 0) {
       const FieldDescriptor* extension = message_descriptor->extension(0);
       if (extension->is_extension() &&
@@ -251,7 +253,7 @@ PyObject* _FindExtensionByName(ExtensionDict* self, PyObject* arg) {
       }
     }
   }
-  if (message_extension == NULL) {
+  if (message_extension == nullptr) {
     Py_RETURN_NONE;
   }
 
@@ -259,15 +261,15 @@ PyObject* _FindExtensionByName(ExtensionDict* self, PyObject* arg) {
 }
 
 PyObject* _FindExtensionByNumber(ExtensionDict* self, PyObject* arg) {
-  int64 number = PyLong_AsLong(arg);
+  int64_t number = PyLong_AsLong(arg);
   if (number == -1 && PyErr_Occurred()) {
-    return NULL;
+    return nullptr;
   }
 
   PyDescriptorPool* pool = cmessage::GetFactoryForMessage(self->parent)->pool;
   const FieldDescriptor* message_extension = pool->pool->FindExtensionByNumber(
       self->parent->message->GetDescriptor(), number);
-  if (message_extension == NULL) {
+  if (message_extension == nullptr) {
     Py_RETURN_NONE;
   }
 
@@ -306,8 +308,8 @@ static int Contains(PyObject* _self, PyObject* key) {
 ExtensionDict* NewExtensionDict(CMessage *parent) {
   ExtensionDict* self = reinterpret_cast<ExtensionDict*>(
       PyType_GenericAlloc(&ExtensionDict_Type, 0));
-  if (self == NULL) {
-    return NULL;
+  if (self == nullptr) {
+    return nullptr;
   }
 
   Py_INCREF(parent);
@@ -329,7 +331,7 @@ static PyObject* RichCompare(ExtensionDict* self, PyObject* other, int opid) {
   }
   bool equals = false;
   if (PyObject_TypeCheck(other, &ExtensionDict_Type)) {
-    equals = self->parent == reinterpret_cast<ExtensionDict*>(other)->parent;;
+    equals = self->parent == reinterpret_cast<ExtensionDict*>(other)->parent;
   }
   if (equals ^ (opid == Py_EQ)) {
     Py_RETURN_FALSE;
@@ -339,12 +341,12 @@ static PyObject* RichCompare(ExtensionDict* self, PyObject* other, int opid) {
 }
 static PySequenceMethods SeqMethods = {
     (lenfunc)len,          // sq_length
-    0,                     // sq_concat
-    0,                     // sq_repeat
-    0,                     // sq_item
-    0,                     // sq_slice
-    0,                     // sq_ass_item
-    0,                     // sq_ass_slice
+    nullptr,               // sq_concat
+    nullptr,               // sq_repeat
+    nullptr,               // sq_item
+    nullptr,               // sq_slice
+    nullptr,               // sq_ass_item
+    nullptr,               // sq_ass_slice
     (objobjproc)Contains,  // sq_contains
 };
 
@@ -359,48 +361,52 @@ static PyMethodDef Methods[] = {
     EDMETHOD(_FindExtensionByName, METH_O, "Finds an extension by name."),
     EDMETHOD(_FindExtensionByNumber, METH_O,
              "Finds an extension by field number."),
-    {NULL, NULL},
+    {nullptr, nullptr},
 };
 
 }  // namespace extension_dict
 
 PyTypeObject ExtensionDict_Type = {
-    PyVarObject_HEAD_INIT(&PyType_Type, 0)     //
-    FULL_MODULE_NAME ".ExtensionDict",         // tp_name
-    sizeof(ExtensionDict),                     // tp_basicsize
-    0,                                         //  tp_itemsize
-    (destructor)extension_dict::dealloc,       //  tp_dealloc
-    0,                                         //  tp_print
-    0,                                         //  tp_getattr
-    0,                                         //  tp_setattr
-    0,                                         //  tp_compare
-    0,                                         //  tp_repr
-    0,                                         //  tp_as_number
+    PyVarObject_HEAD_INIT(&PyType_Type, 0)  //
+    FULL_MODULE_NAME ".ExtensionDict",      // tp_name
+    sizeof(ExtensionDict),                  // tp_basicsize
+    0,                                      //  tp_itemsize
+    (destructor)extension_dict::dealloc,    //  tp_dealloc
+#if PY_VERSION_HEX < 0x03080000
+    nullptr,  // tp_print
+#else
+    0,  // tp_vectorcall_offset
+#endif
+    nullptr,                                   //  tp_getattr
+    nullptr,                                   //  tp_setattr
+    nullptr,                                   //  tp_compare
+    nullptr,                                   //  tp_repr
+    nullptr,                                   //  tp_as_number
     &extension_dict::SeqMethods,               //  tp_as_sequence
     &extension_dict::MpMethods,                //  tp_as_mapping
     PyObject_HashNotImplemented,               //  tp_hash
-    0,                                         //  tp_call
-    0,                                         //  tp_str
-    0,                                         //  tp_getattro
-    0,                                         //  tp_setattro
-    0,                                         //  tp_as_buffer
+    nullptr,                                   //  tp_call
+    nullptr,                                   //  tp_str
+    nullptr,                                   //  tp_getattro
+    nullptr,                                   //  tp_setattro
+    nullptr,                                   //  tp_as_buffer
     Py_TPFLAGS_DEFAULT,                        //  tp_flags
     "An extension dict",                       //  tp_doc
-    0,                                         //  tp_traverse
-    0,                                         //  tp_clear
+    nullptr,                                   //  tp_traverse
+    nullptr,                                   //  tp_clear
     (richcmpfunc)extension_dict::RichCompare,  //  tp_richcompare
     0,                                         //  tp_weaklistoffset
     extension_dict::GetIter,                   //  tp_iter
-    0,                                         //  tp_iternext
+    nullptr,                                   //  tp_iternext
     extension_dict::Methods,                   //  tp_methods
-    0,                                         //  tp_members
-    0,                                         //  tp_getset
-    0,                                         //  tp_base
-    0,                                         //  tp_dict
-    0,                                         //  tp_descr_get
-    0,                                         //  tp_descr_set
+    nullptr,                                   //  tp_members
+    nullptr,                                   //  tp_getset
+    nullptr,                                   //  tp_base
+    nullptr,                                   //  tp_dict
+    nullptr,                                   //  tp_descr_get
+    nullptr,                                   //  tp_descr_set
     0,                                         //  tp_dictoffset
-    0,                                         //  tp_init
+    nullptr,                                   //  tp_init
 };
 
 PyObject* IterNext(PyObject* _self) {
@@ -437,37 +443,41 @@ PyTypeObject ExtensionIterator_Type = {
     sizeof(extension_dict::ExtensionIterator),  //  tp_basicsize
     0,                                          //  tp_itemsize
     extension_dict::DeallocExtensionIterator,   //  tp_dealloc
-    0,                                          //  tp_print
-    0,                                          //  tp_getattr
-    0,                                          //  tp_setattr
-    0,                                          //  tp_compare
-    0,                                          //  tp_repr
-    0,                                          //  tp_as_number
-    0,                                          //  tp_as_sequence
-    0,                                          //  tp_as_mapping
-    0,                                          //  tp_hash
-    0,                                          //  tp_call
-    0,                                          //  tp_str
-    0,                                          //  tp_getattro
-    0,                                          //  tp_setattro
-    0,                                          //  tp_as_buffer
-    Py_TPFLAGS_DEFAULT,                         //  tp_flags
-    "A scalar map iterator",                    //  tp_doc
-    0,                                          //  tp_traverse
-    0,                                          //  tp_clear
-    0,                                          //  tp_richcompare
-    0,                                          //  tp_weaklistoffset
-    PyObject_SelfIter,                          //  tp_iter
-    IterNext,                                   //  tp_iternext
-    0,                                          //  tp_methods
-    0,                                          //  tp_members
-    0,                                          //  tp_getset
-    0,                                          //  tp_base
-    0,                                          //  tp_dict
-    0,                                          //  tp_descr_get
-    0,                                          //  tp_descr_set
-    0,                                          //  tp_dictoffset
-    0,                                          //  tp_init
+#if PY_VERSION_HEX < 0x03080000
+    nullptr,  // tp_print
+#else
+    0,  // tp_vectorcall_offset
+#endif
+    nullptr,                  //  tp_getattr
+    nullptr,                  //  tp_setattr
+    nullptr,                  //  tp_compare
+    nullptr,                  //  tp_repr
+    nullptr,                  //  tp_as_number
+    nullptr,                  //  tp_as_sequence
+    nullptr,                  //  tp_as_mapping
+    nullptr,                  //  tp_hash
+    nullptr,                  //  tp_call
+    nullptr,                  //  tp_str
+    nullptr,                  //  tp_getattro
+    nullptr,                  //  tp_setattro
+    nullptr,                  //  tp_as_buffer
+    Py_TPFLAGS_DEFAULT,       //  tp_flags
+    "A scalar map iterator",  //  tp_doc
+    nullptr,                  //  tp_traverse
+    nullptr,                  //  tp_clear
+    nullptr,                  //  tp_richcompare
+    0,                        //  tp_weaklistoffset
+    PyObject_SelfIter,        //  tp_iter
+    IterNext,                 //  tp_iternext
+    nullptr,                  //  tp_methods
+    nullptr,                  //  tp_members
+    nullptr,                  //  tp_getset
+    nullptr,                  //  tp_base
+    nullptr,                  //  tp_dict
+    nullptr,                  //  tp_descr_get
+    nullptr,                  //  tp_descr_set
+    0,                        //  tp_dictoffset
+    nullptr,                  //  tp_init
 };
 }  // namespace python
 }  // namespace protobuf
