@@ -21,6 +21,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
       :nitems, :iter_for_reverse_each, :indexes, :append, :prepend]
     arr_methods -= [:union, :difference, :filter!]
     arr_methods -= [:intersection, :deconstruct] # ruby 2.7 methods we can ignore
+    arr_methods -= [:intersect?] # ruby 3.1 methods we can ignore
     arr_methods.each do |method_name|
       assert m.repeated_string.respond_to?(method_name) == true, "does not respond to #{method_name}"
     end
@@ -35,18 +36,22 @@ class RepeatedFieldTest < Test::Unit::TestCase
     end
 
     fill_test_msg(m)
-    assert_equal -10, m.repeated_int32.first
-    assert_equal -1_000_000, m.repeated_int64.first
+    assert_equal( -10, m.repeated_int32.first )
+    assert_equal( -1_000_000, m.repeated_int64.first )
     assert_equal 10, m.repeated_uint32.first
     assert_equal 1_000_000, m.repeated_uint64.first
     assert_equal true, m.repeated_bool.first
-    assert_equal -1.01,  m.repeated_float.first.round(2)
-    assert_equal -1.0000000000001, m.repeated_double.first
+    assert_equal( -1.01,  m.repeated_float.first.round(2) )
+    assert_equal( -1.0000000000001, m.repeated_double.first )
     assert_equal 'foo', m.repeated_string.first
     assert_equal "bar".encode!('ASCII-8BIT'), m.repeated_bytes.first
     assert_equal TestMessage2.new(:foo => 1), m.repeated_msg.first
     assert_equal :A, m.repeated_enum.first
 
+    err = assert_raises(ArgumentError) do
+      m.repeated_int32.first(-1)
+    end
+    assert_equal "negative array size", err.message
     assert_equal [], m.repeated_int32.first(0)
     assert_equal [-10], m.repeated_int32.first(1)
     assert_equal [-10, -11], m.repeated_int32.first(2)
@@ -60,17 +65,26 @@ class RepeatedFieldTest < Test::Unit::TestCase
       assert_nil m.send(field_name).first
     end
     fill_test_msg(m)
-    assert_equal -11, m.repeated_int32.last
-    assert_equal -1_000_001, m.repeated_int64.last
+    assert_equal( -11, m.repeated_int32.last )
+    assert_equal( -1_000_001, m.repeated_int64.last )
     assert_equal 11, m.repeated_uint32.last
     assert_equal 1_000_001, m.repeated_uint64.last
     assert_equal false, m.repeated_bool.last
-    assert_equal -1.02, m.repeated_float.last.round(2)
-    assert_equal -1.0000000000002, m.repeated_double.last
+    assert_equal( -1.02, m.repeated_float.last.round(2) )
+    assert_equal( -1.0000000000002, m.repeated_double.last )
     assert_equal 'bar', m.repeated_string.last
     assert_equal "foo".encode!('ASCII-8BIT'), m.repeated_bytes.last
     assert_equal TestMessage2.new(:foo => 2), m.repeated_msg.last
     assert_equal :B, m.repeated_enum.last
+
+    err = assert_raises(ArgumentError) do
+      m.repeated_int32.last(-1)
+    end
+    assert_equal "negative array size", err.message
+    assert_equal [], m.repeated_int32.last(0)
+    assert_equal [-11], m.repeated_int32.last(1)
+    assert_equal [-10, -11], m.repeated_int32.last(2)
+    assert_equal [-10, -11], m.repeated_int32.last(3)
   end
 
 
@@ -81,20 +95,20 @@ class RepeatedFieldTest < Test::Unit::TestCase
     end
     fill_test_msg(m)
 
-    assert_equal -11, m.repeated_int32.pop
-    assert_equal -10, m.repeated_int32.pop
-    assert_equal -1_000_001, m.repeated_int64.pop
-    assert_equal -1_000_000, m.repeated_int64.pop
+    assert_equal( -11, m.repeated_int32.pop )
+    assert_equal( -10, m.repeated_int32.pop )
+    assert_equal( -1_000_001, m.repeated_int64.pop )
+    assert_equal( -1_000_000, m.repeated_int64.pop )
     assert_equal 11, m.repeated_uint32.pop
     assert_equal 10, m.repeated_uint32.pop
     assert_equal 1_000_001, m.repeated_uint64.pop
     assert_equal 1_000_000, m.repeated_uint64.pop
     assert_equal false, m.repeated_bool.pop
     assert_equal true, m.repeated_bool.pop
-    assert_equal -1.02,  m.repeated_float.pop.round(2)
-    assert_equal -1.01,  m.repeated_float.pop.round(2)
-    assert_equal -1.0000000000002, m.repeated_double.pop
-    assert_equal -1.0000000000001, m.repeated_double.pop
+    assert_equal( -1.02,  m.repeated_float.pop.round(2) )
+    assert_equal( -1.01,  m.repeated_float.pop.round(2) )
+    assert_equal( -1.0000000000002, m.repeated_double.pop )
+    assert_equal( -1.0000000000001, m.repeated_double.pop )
     assert_equal 'bar', m.repeated_string.pop
     assert_equal 'foo', m.repeated_string.pop
     assert_equal "foo".encode!('ASCII-8BIT'), m.repeated_bytes.pop
@@ -161,6 +175,51 @@ class RepeatedFieldTest < Test::Unit::TestCase
     check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
       arr[0..2]
     end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      arr[0..5]
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      arr[0..-1]
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      arr[0..-3]
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      arr[0...-1] # Exclusive range
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      arr[0...-3] # Exclusive range
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      arr[-2..-1]
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      arr[-5..-1]
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      # Infinite range; introduce in Ruby 2.7.
+      if Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('2.7')
+        eval "arr[0..]"
+      end
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      # Beginless range; introduced in Ruby 2.7.
+      if Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('2.7')
+        eval "arr[..-1]"
+      end
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      # Infinite range; introduce in Ruby 2.7.
+      if Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('2.7')
+        eval "arr[0...]" # Exclusive range
+      end
+    end
+    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
+      # Beginless range; introduced in Ruby 2.7.
+      if Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('2.7')
+        eval "arr[...-1]" # Exclusive range
+      end
+    end
     check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
       arr[-1, 1]
     end
@@ -339,18 +398,6 @@ class RepeatedFieldTest < Test::Unit::TestCase
     end
   end
 
-  def test_compact!
-    m = TestMessage.new
-    m.repeated_msg << TestMessage2.new(:foo => 1)
-    m.repeated_msg << nil
-    m.repeated_msg << TestMessage2.new(:foo => 2)
-    reference_arr = m.repeated_string.to_a
-
-    check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
-      arr.compact!
-    end
-  end
-
   def test_delete
     m = TestMessage.new
     reference_arr = %w(foo bar baz)
@@ -498,11 +545,8 @@ class RepeatedFieldTest < Test::Unit::TestCase
   def test_shuffle!
     m = TestMessage.new
     m.repeated_string += %w(foo bar baz)
-    orig_repeated_string = m.repeated_string.clone
     result = m.repeated_string.shuffle!
     assert_equal m.repeated_string, result
-    # NOTE: sometimes it doesn't change the order...
-    # assert_not_equal m.repeated_string.to_a, orig_repeated_string.to_a
   end
 
   def test_slice!
@@ -653,6 +697,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
       value :A, 1
       value :B, 2
       value :C, 3
+      value :v0, 4
     end
   end
 
