@@ -33,10 +33,25 @@
 // This is a dummy code generator plugin used by
 // command_line_interface_unittest.
 
-#include <string>
 #include <stdlib.h>
-#include <google/protobuf/compiler/plugin.h>
+
+#include <string>
+
 #include <google/protobuf/compiler/mock_code_generator.h>
+#include <google/protobuf/compiler/plugin.h>
+
+namespace google {
+namespace protobuf {
+namespace compiler {
+
+int ProtobufMain(int argc, char* argv[]) {
+  MockCodeGenerator generator("test_plugin");
+  return PluginMain(argc, argv, &generator);
+}
+
+}  // namespace compiler
+}  // namespace protobuf
+}  // namespace google
 
 int main(int argc, char* argv[]) {
 #ifdef _MSC_VER
@@ -44,7 +59,5 @@ int main(int argc, char* argv[]) {
   // please.
   _set_abort_behavior(0, ~0);
 #endif  // !_MSC_VER
-
-  google::protobuf::compiler::MockCodeGenerator generator("test_plugin");
-  return google::protobuf::compiler::PluginMain(argc, argv, &generator);
+  return google::protobuf::compiler::ProtobufMain(argc, argv);
 }
