@@ -37,17 +37,16 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include <atomic>
 #include <limits>
 #include <memory>
 
 #include <google/protobuf/stubs/logging.h>
 #include <google/protobuf/stubs/common.h>
-#include <google/protobuf/stubs/logging.h>
 #include <google/protobuf/testing/file.h>
 #include <google/protobuf/testing/file.h>
+#include <google/protobuf/any.pb.h>
 #include <google/protobuf/map_unittest.pb.h>
-#include <google/protobuf/test_util.h>
-#include <google/protobuf/test_util2.h>
 #include <google/protobuf/unittest.pb.h>
 #include <google/protobuf/unittest_mset.pb.h>
 #include <google/protobuf/unittest_mset_wire_format.pb.h>
@@ -55,19 +54,31 @@
 #include <google/protobuf/io/tokenizer.h>
 #include <google/protobuf/io/zero_copy_stream_impl.h>
 #include <google/protobuf/stubs/strutil.h>
-#include <google/protobuf/stubs/substitute.h>
+#include <gmock/gmock.h>
 #include <google/protobuf/testing/googletest.h>
 #include <gtest/gtest.h>
+#include <google/protobuf/stubs/logging.h>
+#include <google/protobuf/stubs/substitute.h>
+#include <google/protobuf/test_util.h>
+#include <google/protobuf/test_util2.h>
 
 
+// Must be included last.
 #include <google/protobuf/port_def.inc>
 
 namespace google {
 namespace protobuf {
 
+namespace internal {
+// Controls insertion of DEBUG_STRING_SILENT_MARKER.
+extern PROTOBUF_EXPORT std::atomic<bool> enable_debug_text_format_marker;
+}  // namespace internal
+
 // Can't use an anonymous namespace here due to brokenness of Tru64 compiler.
 namespace text_format_unittest {
 
+using ::google::protobuf::internal::kDebugStringSilentMarker;
+
 // A basic string with different escapable characters for testing.
 const std::string kEscapeTestString =
     "\"A string with ' characters \n and \r newlines and \t tabs and \001 "
@@ -85,21 +96,21 @@ class TextFormatTest : public testing::Test {
         TestUtil::GetTestDataPath(
             "net/proto2/internal/"
             "testdata/text_format_unittest_data_oneof_implemented.txt"),
-        &static_proto_debug_string_, true));
-    CleanStringLineEndings(&static_proto_debug_string_, false);
+        &static_proto_text_format_, true));
+    CleanStringLineEndings(&static_proto_text_format_, false);
   }
 
-  TextFormatTest() : proto_debug_string_(static_proto_debug_string_) {}
+  TextFormatTest() : proto_text_format_(static_proto_text_format_) {}
 
  protected:
-  // Debug string read from text_format_unittest_data.txt.
-  const std::string proto_debug_string_;
+  // Text format read from text_format_unittest_data.txt.
+  const std::string proto_text_format_;
   unittest::TestAllTypes proto_;
 
  private:
-  static std::string static_proto_debug_string_;
+  static std::string static_proto_text_format_;
 };
-std::string TextFormatTest::static_proto_debug_string_;
+std::string TextFormatTest::static_proto_text_format_;
 
 class TextFormatExtensionsTest : public testing::Test {
  public:
@@ -107,31 +118,34 @@ class TextFormatExtensionsTest : public testing::Test {
     GOOGLE_CHECK_OK(File::GetContents(
         TestUtil::GetTestDataPath("net/proto2/internal/testdata/"
                                   "text_format_unittest_extensions_data.txt"),
-        &static_proto_debug_string_, true));
-    CleanStringLineEndings(&static_proto_debug_string_, false);
+        &static_proto_text_format_, true));
+    CleanStringLineEndings(&static_proto_text_format_, false);
   }
 
-  TextFormatExtensionsTest()
-      : proto_debug_string_(static_proto_debug_string_) {}
+  TextFormatExtensionsTest() : proto_text_format_(static_proto_text_format_) {}
 
  protected:
   // Debug string read from text_format_unittest_data.txt.
-  const std::string proto_debug_string_;
+  const std::string proto_text_format_;
   unittest::TestAllExtensions proto_;
 
  private:
-  static std::string static_proto_debug_string_;
+  static std::string static_proto_text_format_;
 };
-std::string TextFormatExtensionsTest::static_proto_debug_string_;
+std::string TextFormatExtensionsTest::static_proto_text_format_;
 
 TEST_F(TextFormatTest, Basic) {
   TestUtil::SetAllFields(&proto_);
-  EXPECT_EQ(proto_debug_string_, proto_.DebugString());
+  std::string actual_proto_text_format;
+  TextFormat::PrintToString(proto_, &actual_proto_text_format);
+  EXPECT_EQ(actual_proto_text_format, proto_text_format_);
 }
 
 TEST_F(TextFormatExtensionsTest, Extensions) {
   TestUtil::SetAllExtensions(&proto_);
-  EXPECT_EQ(proto_debug_string_, proto_.DebugString());
+  std::string actual_proto_text_format;
+  TextFormat::PrintToString(proto_, &actual_proto_text_format);
+  EXPECT_EQ(actual_proto_text_format, proto_text_format_);
 }
 
 TEST_F(TextFormatTest, ShortDebugString) {
@@ -140,11 +154,12 @@ TEST_F(TextFormatTest, ShortDebugString) {
   proto_.mutable_optional_nested_message()->set_bb(2);
   proto_.mutable_optional_foreign_message();
 
-  EXPECT_EQ(
-      "optional_int32: 1 optional_string: \"hello\" "
-      "optional_nested_message { bb: 2 } "
-      "optional_foreign_message { }",
-      proto_.ShortDebugString());
+  EXPECT_EQ(proto_.ShortDebugString(),
+            StrCat("optional_int32: ", kDebugStringSilentMarker,
+                         "1 "
+                         "optional_string: \"hello\" "
+                         "optional_nested_message { bb: 2 } "
+                         "optional_foreign_message { }"));
 }
 
 TEST_F(TextFormatTest, ShortPrimitiveRepeateds) {
@@ -214,7 +229,8 @@ TEST_F(TextFormatTest, StringEscape) {
 
   // Hardcode a correct value to test against.
   std::string correct_string =
-      "optional_string: " + kEscapeTestStringEscaped + "\n";
+      StrCat("optional_string: ", kDebugStringSilentMarker,
+                   kEscapeTestStringEscaped, "\n");
 
   // Compare.
   EXPECT_EQ(correct_string, debug_string);
@@ -222,8 +238,8 @@ TEST_F(TextFormatTest, StringEscape) {
   // the protocol buffer contains no UTF-8 text.
   EXPECT_EQ(correct_string, utf8_debug_string);
 
-  std::string expected_short_debug_string =
-      "optional_string: " + kEscapeTestStringEscaped;
+  std::string expected_short_debug_string = StrCat(
+      "optional_string: ", kDebugStringSilentMarker, kEscapeTestStringEscaped);
   EXPECT_EQ(expected_short_debug_string, proto_.ShortDebugString());
 }
 
@@ -238,19 +254,13 @@ TEST_F(TextFormatTest, Utf8DebugString) {
 
   // Hardcode a correct value to test against.
   std::string correct_utf8_string =
-      "optional_string: "
-      "\"\350\260\267\346\255\214\""
-      "\n"
-      "optional_bytes: "
-      "\"\\350\\260\\267\\346\\255\\214\""
-      "\n";
+      StrCat("optional_string: ", kDebugStringSilentMarker,
+                   "\"\350\260\267\346\255\214\"\n"
+                   "optional_bytes: \"\\350\\260\\267\\346\\255\\214\"\n");
   std::string correct_string =
-      "optional_string: "
-      "\"\\350\\260\\267\\346\\255\\214\""
-      "\n"
-      "optional_bytes: "
-      "\"\\350\\260\\267\\346\\255\\214\""
-      "\n";
+      StrCat("optional_string: ", kDebugStringSilentMarker,
+                   "\"\\350\\260\\267\\346\\255\\214\"\n"
+                   "optional_bytes: \"\\350\\260\\267\\346\\255\\214\"\n");
 
   // Compare.
   EXPECT_EQ(correct_utf8_string, utf8_debug_string);
@@ -273,18 +283,18 @@ TEST_F(TextFormatTest, PrintUnknownFields) {
   unknown_fields->AddVarint(8, 2);
   unknown_fields->AddVarint(8, 3);
 
-  EXPECT_EQ(
-      "5: 1\n"
-      "5: 0x00000002\n"
-      "5: 0x0000000000000003\n"
-      "5: \"4\"\n"
-      "5 {\n"
-      "  10: 5\n"
-      "}\n"
-      "8: 1\n"
-      "8: 2\n"
-      "8: 3\n",
-      message.DebugString());
+  EXPECT_EQ(StrCat("5: ", kDebugStringSilentMarker,
+                         "1\n"
+                         "5: 0x00000002\n"
+                         "5: 0x0000000000000003\n"
+                         "5: \"4\"\n"
+                         "5 {\n"
+                         "  10: 5\n"
+                         "}\n"
+                         "8: 1\n"
+                         "8: 2\n"
+                         "8: 3\n"),
+            message.DebugString());
 }
 
 TEST_F(TextFormatTest, PrintUnknownFieldsHidden) {
@@ -346,14 +356,13 @@ TEST_F(TextFormatTest, PrintUnknownMessage) {
   UnknownFieldSet unknown_fields;
   EXPECT_TRUE(unknown_fields.ParseFromString(data));
   EXPECT_TRUE(TextFormat::PrintUnknownFieldsToString(unknown_fields, &text));
-  EXPECT_EQ(
-      "44: \"abc\"\n"
-      "44: \"def\"\n"
-      "44: \"\"\n"
-      "48 {\n"
-      "  1: 123\n"
-      "}\n",
-      text);
+  // Field 44 and 48 can be printed in any order.
+  EXPECT_THAT(text, testing::HasSubstr("44: \"abc\"\n"
+                                       "44: \"def\"\n"
+                                       "44: \"\"\n"));
+  EXPECT_THAT(text, testing::HasSubstr("48 {\n"
+                                       "  1: 123\n"
+                                       "}\n"));
 }
 
 TEST_F(TextFormatTest, PrintDeeplyNestedUnknownMessage) {
@@ -449,7 +458,7 @@ TEST_F(TextFormatTest, PrintBufferTooSmall) {
 // A printer that appends 'u' to all unsigned int32.
 class CustomUInt32FieldValuePrinter : public TextFormat::FieldValuePrinter {
  public:
-  virtual std::string PrintUInt32(uint32 val) const {
+  std::string PrintUInt32(uint32_t val) const override {
     return StrCat(FieldValuePrinter::PrintUInt32(val), "u");
   }
 };
@@ -473,7 +482,7 @@ TEST_F(TextFormatTest, DefaultCustomFieldPrinter) {
 
 class CustomInt32FieldValuePrinter : public TextFormat::FieldValuePrinter {
  public:
-  virtual std::string PrintInt32(int32 val) const {
+  std::string PrintInt32(int32_t val) const override {
     return StrCat("value-is(", FieldValuePrinter::PrintInt32(val), ")");
   }
 };
@@ -509,7 +518,7 @@ TEST_F(TextFormatTest, FieldSpecificCustomPrinterRegisterSameFieldTwice) {
 TEST_F(TextFormatTest, ErrorCasesRegisteringFieldValuePrinterShouldFail) {
   protobuf_unittest::TestAllTypes message;
   TextFormat::Printer printer;
-  // NULL printer.
+  // nullptr printer.
   EXPECT_FALSE(printer.RegisterFieldValuePrinter(
       message.GetDescriptor()->FindFieldByName("optional_int32"),
       static_cast<const TextFormat::FieldValuePrinter*>(nullptr)));
@@ -518,20 +527,20 @@ TEST_F(TextFormatTest, ErrorCasesRegisteringFieldValuePrinterShouldFail) {
       static_cast<const TextFormat::FastFieldValuePrinter*>(nullptr)));
   // Because registration fails, the ownership of this printer is never taken.
   TextFormat::FieldValuePrinter my_field_printer;
-  // NULL field
+  // nullptr field
   EXPECT_FALSE(printer.RegisterFieldValuePrinter(nullptr, &my_field_printer));
 }
 
 class CustomMessageFieldValuePrinter : public TextFormat::FieldValuePrinter {
  public:
-  virtual std::string PrintInt32(int32 v) const {
+  std::string PrintInt32(int32_t v) const override {
     return StrCat(FieldValuePrinter::PrintInt32(v), "  # x",
                         strings::Hex(v));
   }
 
-  virtual std::string PrintMessageStart(const Message& message, int field_index,
-                                        int field_count,
-                                        bool single_line_mode) const {
+  std::string PrintMessageStart(const Message& message, int field_index,
+                                int field_count,
+                                bool single_line_mode) const override {
     if (single_line_mode) {
       return " { ";
     }
@@ -622,9 +631,9 @@ TEST_F(TextFormatTest, CustomPrinterForMessageContent) {
 
 class CustomMultilineCommentPrinter : public TextFormat::FieldValuePrinter {
  public:
-  virtual std::string PrintMessageStart(const Message& message, int field_index,
-                                        int field_count,
-                                        bool single_line_comment) const {
+  std::string PrintMessageStart(const Message& message, int field_index,
+                                int field_count,
+                                bool single_line_comment) const override {
     return StrCat(" {  # 1\n", "  # 2\n");
   }
 };
@@ -718,7 +727,7 @@ TEST_F(TextFormatTest, CompactRepeatedFieldPrinter) {
       text);
 }
 
-// Print strings into multiple line, with indention. Use this to test
+// Print strings into multiple line, with indentation. Use this to test
 // BaseTextGenerator::Indent and BaseTextGenerator::Outdent.
 class MultilineStringPrinter : public TextFormat::FastFieldValuePrinter {
  public:
@@ -792,15 +801,15 @@ TEST_F(TextFormatTest, CustomMessagePrinter) {
 }
 
 TEST_F(TextFormatTest, ParseBasic) {
-  io::ArrayInputStream input_stream(proto_debug_string_.data(),
-                                    proto_debug_string_.size());
+  io::ArrayInputStream input_stream(proto_text_format_.data(),
+                                    proto_text_format_.size());
   TextFormat::Parse(&input_stream, &proto_);
   TestUtil::ExpectAllFieldsSet(proto_);
 }
 
 TEST_F(TextFormatExtensionsTest, ParseExtensions) {
-  io::ArrayInputStream input_stream(proto_debug_string_.data(),
-                                    proto_debug_string_.size());
+  io::ArrayInputStream input_stream(proto_text_format_.data(),
+                                    proto_text_format_.size());
   TextFormat::Parse(&input_stream, &proto_);
   TestUtil::ExpectAllExtensionsSet(proto_);
 }
@@ -836,12 +845,12 @@ TEST_F(TextFormatTest, PrintUnknownEnumFieldProto3) {
   proto.add_repeated_nested_enum(
       static_cast<proto3_unittest::TestAllTypes::NestedEnum>(-2147483648));
 
-  EXPECT_EQ(
-      "repeated_nested_enum: 10\n"
-      "repeated_nested_enum: -10\n"
-      "repeated_nested_enum: 2147483647\n"
-      "repeated_nested_enum: -2147483648\n",
-      proto.DebugString());
+  EXPECT_EQ(StrCat("repeated_nested_enum: ", kDebugStringSilentMarker,
+                         "10\n"
+                         "repeated_nested_enum: -10\n"
+                         "repeated_nested_enum: 2147483647\n"
+                         "repeated_nested_enum: -2147483648\n"),
+            proto.DebugString());
 }
 
 TEST_F(TextFormatTest, ParseUnknownEnumFieldProto3) {
@@ -1069,13 +1078,8 @@ static std::string RemoveRedundantZeros(std::string text) {
 TEST_F(TextFormatTest, PrintExotic) {
   unittest::TestAllTypes message;
 
-  // Note:  In C, a negative integer literal is actually the unary negation
-  //   operator being applied to a positive integer literal, and
-  //   9223372036854775808 is outside the range of int64.  However, it is not
-  //   outside the range of uint64.  Confusingly, this means that everything
-  //   works if we make the literal unsigned, even though we are negating it.
-  message.add_repeated_int64(-PROTOBUF_ULONGLONG(9223372036854775808));
-  message.add_repeated_uint64(PROTOBUF_ULONGLONG(18446744073709551615));
+  message.add_repeated_int64(int64_t{-9223372036854775807} - 1);
+  message.add_repeated_uint64(uint64_t{18446744073709551615u});
   message.add_repeated_double(123.456);
   message.add_repeated_double(1.23e21);
   message.add_repeated_double(1.23e-18);
@@ -1091,7 +1095,7 @@ TEST_F(TextFormatTest, PrintExotic) {
   //   seemed to trigger an odd case on MinGW/GCC 3.4.5 where GCC's parsing of
   //   the value differed from strtod()'s parsing.  That is to say, the
   //   following assertion fails on MinGW:
-  //     assert(1.23e22 == strtod("1.23e22", NULL));
+  //     assert(1.23e22 == strtod("1.23e22", nullptr));
   //   As a result, SimpleDtoa() would print the value as
   //   "1.2300000000000001e+22" to make sure strtod() produce the exact same
   //   result.  Our goal is to test runtime parsing, not compile-time parsing,
@@ -1099,19 +1103,20 @@ TEST_F(TextFormatTest, PrintExotic) {
   //   have this problem, so we switched to that instead.
 
   EXPECT_EQ(
-      "repeated_int64: -9223372036854775808\n"
-      "repeated_uint64: 18446744073709551615\n"
-      "repeated_double: 123.456\n"
-      "repeated_double: 1.23e+21\n"
-      "repeated_double: 1.23e-18\n"
-      "repeated_double: inf\n"
-      "repeated_double: -inf\n"
-      "repeated_double: nan\n"
-      "repeated_double: nan\n"
-      "repeated_double: nan\n"
-      "repeated_double: nan\n"
-      "repeated_string: "
-      "\"\\000\\001\\007\\010\\014\\n\\r\\t\\013\\\\\\'\\\"\"\n",
+      StrCat("repeated_int64: ", kDebugStringSilentMarker,
+                   "-9223372036854775808\n"
+                   "repeated_uint64: 18446744073709551615\n"
+                   "repeated_double: 123.456\n"
+                   "repeated_double: 1.23e+21\n"
+                   "repeated_double: 1.23e-18\n"
+                   "repeated_double: inf\n"
+                   "repeated_double: -inf\n"
+                   "repeated_double: nan\n"
+                   "repeated_double: nan\n"
+                   "repeated_double: nan\n"
+                   "repeated_double: nan\n"
+                   "repeated_string: "
+                   "\"\\000\\001\\007\\010\\014\\n\\r\\t\\013\\\\\\'\\\"\"\n"),
       RemoveRedundantZeros(message.DebugString()));
 }
 
@@ -1158,47 +1163,47 @@ TEST_F(TextFormatTest, PrintFloatPrecision) {
   message.add_repeated_double(1.2345678987654e100);
   message.add_repeated_double(1.23456789876543e100);
 
-  EXPECT_EQ(
-      "repeated_float: 1\n"
-      "repeated_float: 1.2\n"
-      "repeated_float: 1.23\n"
-      "repeated_float: 1.234\n"
-      "repeated_float: 1.2345\n"
-      "repeated_float: 1.23456\n"
-      "repeated_float: 1.2e+10\n"
-      "repeated_float: 1.23e+10\n"
-      "repeated_float: 1.234e+10\n"
-      "repeated_float: 1.2345e+10\n"
-      "repeated_float: 1.23456e+10\n"
-      "repeated_double: 1.2\n"
-      "repeated_double: 1.23\n"
-      "repeated_double: 1.234\n"
-      "repeated_double: 1.2345\n"
-      "repeated_double: 1.23456\n"
-      "repeated_double: 1.234567\n"
-      "repeated_double: 1.2345678\n"
-      "repeated_double: 1.23456789\n"
-      "repeated_double: 1.234567898\n"
-      "repeated_double: 1.2345678987\n"
-      "repeated_double: 1.23456789876\n"
-      "repeated_double: 1.234567898765\n"
-      "repeated_double: 1.2345678987654\n"
-      "repeated_double: 1.23456789876543\n"
-      "repeated_double: 1.2e+100\n"
-      "repeated_double: 1.23e+100\n"
-      "repeated_double: 1.234e+100\n"
-      "repeated_double: 1.2345e+100\n"
-      "repeated_double: 1.23456e+100\n"
-      "repeated_double: 1.234567e+100\n"
-      "repeated_double: 1.2345678e+100\n"
-      "repeated_double: 1.23456789e+100\n"
-      "repeated_double: 1.234567898e+100\n"
-      "repeated_double: 1.2345678987e+100\n"
-      "repeated_double: 1.23456789876e+100\n"
-      "repeated_double: 1.234567898765e+100\n"
-      "repeated_double: 1.2345678987654e+100\n"
-      "repeated_double: 1.23456789876543e+100\n",
-      RemoveRedundantZeros(message.DebugString()));
+  EXPECT_EQ(StrCat("repeated_float: ", kDebugStringSilentMarker,
+                         "1\n"
+                         "repeated_float: 1.2\n"
+                         "repeated_float: 1.23\n"
+                         "repeated_float: 1.234\n"
+                         "repeated_float: 1.2345\n"
+                         "repeated_float: 1.23456\n"
+                         "repeated_float: 1.2e+10\n"
+                         "repeated_float: 1.23e+10\n"
+                         "repeated_float: 1.234e+10\n"
+                         "repeated_float: 1.2345e+10\n"
+                         "repeated_float: 1.23456e+10\n"
+                         "repeated_double: 1.2\n"
+                         "repeated_double: 1.23\n"
+                         "repeated_double: 1.234\n"
+                         "repeated_double: 1.2345\n"
+                         "repeated_double: 1.23456\n"
+                         "repeated_double: 1.234567\n"
+                         "repeated_double: 1.2345678\n"
+                         "repeated_double: 1.23456789\n"
+                         "repeated_double: 1.234567898\n"
+                         "repeated_double: 1.2345678987\n"
+                         "repeated_double: 1.23456789876\n"
+                         "repeated_double: 1.234567898765\n"
+                         "repeated_double: 1.2345678987654\n"
+                         "repeated_double: 1.23456789876543\n"
+                         "repeated_double: 1.2e+100\n"
+                         "repeated_double: 1.23e+100\n"
+                         "repeated_double: 1.234e+100\n"
+                         "repeated_double: 1.2345e+100\n"
+                         "repeated_double: 1.23456e+100\n"
+                         "repeated_double: 1.234567e+100\n"
+                         "repeated_double: 1.2345678e+100\n"
+                         "repeated_double: 1.23456789e+100\n"
+                         "repeated_double: 1.234567898e+100\n"
+                         "repeated_double: 1.2345678987e+100\n"
+                         "repeated_double: 1.23456789876e+100\n"
+                         "repeated_double: 1.234567898765e+100\n"
+                         "repeated_double: 1.2345678987654e+100\n"
+                         "repeated_double: 1.23456789876543e+100\n"),
+            RemoveRedundantZeros(message.DebugString()));
 }
 
 TEST_F(TextFormatTest, AllowPartial) {
@@ -1240,32 +1245,19 @@ TEST_F(TextFormatTest, ParseExotic) {
 
   ASSERT_EQ(2, message.repeated_int32_size());
   EXPECT_EQ(-1, message.repeated_int32(0));
-  // Note:  In C, a negative integer literal is actually the unary negation
-  //   operator being applied to a positive integer literal, and 2147483648 is
-  //   outside the range of int32.  However, it is not outside the range of
-  //   uint32.  Confusingly, this means that everything works if we make the
-  //   literal unsigned, even though we are negating it.
-  EXPECT_EQ(-2147483648u, message.repeated_int32(1));
+  EXPECT_EQ(-2147483648, message.repeated_int32(1));
 
   ASSERT_EQ(2, message.repeated_int64_size());
   EXPECT_EQ(-1, message.repeated_int64(0));
-  // Note:  In C, a negative integer literal is actually the unary negation
-  //   operator being applied to a positive integer literal, and
-  //   9223372036854775808 is outside the range of int64.  However, it is not
-  //   outside the range of uint64.  Confusingly, this means that everything
-  //   works if we make the literal unsigned, even though we are negating it.
-  EXPECT_EQ(-PROTOBUF_ULONGLONG(9223372036854775808),
-            message.repeated_int64(1));
+  EXPECT_EQ(int64_t{-9223372036854775807} - 1, message.repeated_int64(1));
 
   ASSERT_EQ(2, message.repeated_uint32_size());
   EXPECT_EQ(4294967295u, message.repeated_uint32(0));
   EXPECT_EQ(2147483648u, message.repeated_uint32(1));
 
   ASSERT_EQ(2, message.repeated_uint64_size());
-  EXPECT_EQ(PROTOBUF_ULONGLONG(18446744073709551615),
-            message.repeated_uint64(0));
-  EXPECT_EQ(PROTOBUF_ULONGLONG(9223372036854775808),
-            message.repeated_uint64(1));
+  EXPECT_EQ(uint64_t{18446744073709551615u}, message.repeated_uint64(0));
+  EXPECT_EQ(uint64_t{9223372036854775808u}, message.repeated_uint64(1));
 
   ASSERT_EQ(13, message.repeated_double_size());
   EXPECT_EQ(123.0, message.repeated_double(0));
@@ -1409,9 +1401,8 @@ class TextFormatParserTest : public testing::Test {
     parser_.RecordErrorsTo(&error_collector);
     EXPECT_EQ(expected_result, parser_.ParseFromString(input, proto))
         << input << " -> " << proto->DebugString();
-    EXPECT_EQ(
-        StrCat(line) + ":" + StrCat(col) + ": " + message + "\n",
-        error_collector.text_);
+    EXPECT_EQ(StrCat(line, ":", col, ": ", message, "\n"),
+              error_collector.text_);
     parser_.RecordErrorsTo(nullptr);
   }
 
@@ -1426,12 +1417,18 @@ class TextFormatParserTest : public testing::Test {
   }
 
   void ExpectLocation(TextFormat::ParseInfoTree* tree, const Descriptor* d,
-                      const std::string& field_name, int index, int line,
-                      int column) {
-    TextFormat::ParseLocation location =
+                      const std::string& field_name, int index, int start_line,
+                      int start_column, int end_line, int end_column) {
+    TextFormat::ParseLocationRange range =
+        tree->GetLocationRange(d->FindFieldByName(field_name), index);
+    EXPECT_EQ(start_line, range.start.line);
+    EXPECT_EQ(start_column, range.start.column);
+    EXPECT_EQ(end_line, range.end.line);
+    EXPECT_EQ(end_column, range.end.column);
+    TextFormat::ParseLocation start_location =
         tree->GetLocation(d->FindFieldByName(field_name), index);
-    EXPECT_EQ(line, location.line);
-    EXPECT_EQ(column, location.column);
+    EXPECT_EQ(start_line, start_location.line);
+    EXPECT_EQ(start_column, start_location.column);
   }
 
   // An error collector which simply concatenates all its errors into a big
@@ -1439,17 +1436,17 @@ class TextFormatParserTest : public testing::Test {
   class MockErrorCollector : public io::ErrorCollector {
    public:
     MockErrorCollector() {}
-    ~MockErrorCollector() {}
+    ~MockErrorCollector() override {}
 
     std::string text_;
 
     // implements ErrorCollector -------------------------------------
-    void AddError(int line, int column, const std::string& message) {
+    void AddError(int line, int column, const std::string& message) override {
       strings::SubstituteAndAppend(&text_, "$0:$1: $2\n", line + 1, column + 1,
                                 message);
     }
 
-    void AddWarning(int line, int column, const std::string& message) {
+    void AddWarning(int line, int column, const std::string& message) override {
       AddError(line, column, "WARNING:" + message);
     }
   };
@@ -1481,22 +1478,22 @@ TEST_F(TextFormatParserTest, ParseInfoTreeBuilding) {
   ExpectSuccessAndTree(stringData, message.get(), &tree);
 
   // Verify that the tree has the correct positions.
-  ExpectLocation(&tree, d, "optional_int32", -1, 0, 0);
-  ExpectLocation(&tree, d, "optional_int64", -1, 1, 0);
-  ExpectLocation(&tree, d, "optional_double", -1, 2, 2);
+  ExpectLocation(&tree, d, "optional_int32", -1, 0, 0, 0, 17);
+  ExpectLocation(&tree, d, "optional_int64", -1, 1, 0, 1, 17);
+  ExpectLocation(&tree, d, "optional_double", -1, 2, 2, 2, 22);
 
-  ExpectLocation(&tree, d, "repeated_int32", 0, 3, 0);
-  ExpectLocation(&tree, d, "repeated_int32", 1, 4, 0);
+  ExpectLocation(&tree, d, "repeated_int32", 0, 3, 0, 3, 17);
+  ExpectLocation(&tree, d, "repeated_int32", 1, 4, 0, 4, 18);
 
-  ExpectLocation(&tree, d, "optional_nested_message", -1, 5, 0);
-  ExpectLocation(&tree, d, "repeated_nested_message", 0, 8, 0);
-  ExpectLocation(&tree, d, "repeated_nested_message", 1, 11, 0);
+  ExpectLocation(&tree, d, "optional_nested_message", -1, 5, 0, 7, 1);
+  ExpectLocation(&tree, d, "repeated_nested_message", 0, 8, 0, 10, 1);
+  ExpectLocation(&tree, d, "repeated_nested_message", 1, 11, 0, 13, 1);
 
-  // Check for fields not set. For an invalid field, the location returned
-  // should be -1, -1.
-  ExpectLocation(&tree, d, "repeated_int64", 0, -1, -1);
-  ExpectLocation(&tree, d, "repeated_int32", 6, -1, -1);
-  ExpectLocation(&tree, d, "some_unknown_field", -1, -1, -1);
+  // Check for fields not set. For an invalid field, the start and end locations
+  // returned should be -1, -1.
+  ExpectLocation(&tree, d, "repeated_int64", 0, -1, -1, -1, -1);
+  ExpectLocation(&tree, d, "repeated_int32", 6, -1, -1, -1, -1);
+  ExpectLocation(&tree, d, "some_unknown_field", -1, -1, -1, -1, -1);
 
   // Verify inside the nested message.
   const FieldDescriptor* nested_field =
@@ -1504,21 +1501,24 @@ TEST_F(TextFormatParserTest, ParseInfoTreeBuilding) {
 
   TextFormat::ParseInfoTree* nested_tree =
       tree.GetTreeForNested(nested_field, -1);
-  ExpectLocation(nested_tree, nested_field->message_type(), "bb", -1, 6, 2);
+  ExpectLocation(nested_tree, nested_field->message_type(), "bb", -1, 6, 2, 6,
+                 8);
 
   // Verify inside another nested message.
   nested_field = d->FindFieldByName("repeated_nested_message");
   nested_tree = tree.GetTreeForNested(nested_field, 0);
-  ExpectLocation(nested_tree, nested_field->message_type(), "bb", -1, 9, 2);
+  ExpectLocation(nested_tree, nested_field->message_type(), "bb", -1, 9, 2, 9,
+                 8);
 
   nested_tree = tree.GetTreeForNested(nested_field, 1);
-  ExpectLocation(nested_tree, nested_field->message_type(), "bb", -1, 12, 2);
+  ExpectLocation(nested_tree, nested_field->message_type(), "bb", -1, 12, 2, 12,
+                 8);
 
-  // Verify a NULL tree for an unknown nested field.
+  // Verify a nullptr tree for an unknown nested field.
   TextFormat::ParseInfoTree* unknown_nested_tree =
       tree.GetTreeForNested(nested_field, 2);
 
-  EXPECT_EQ(NULL, unknown_nested_tree);
+  EXPECT_EQ(nullptr, unknown_nested_tree);
 }
 
 TEST_F(TextFormatParserTest, ParseFieldValueFromString) {
@@ -1911,17 +1911,164 @@ TEST_F(TextFormatParserTest, SetRecursionLimit) {
 
   input = strings::Substitute(format, input);
   parser_.SetRecursionLimit(100);
-  ExpectMessage(input, "Message is too deep", 1, 908, &message, false);
+  ExpectMessage(input,
+                "Message is too deep, the parser exceeded the configured "
+                "recursion limit of 100.",
+                1, 908, &message, false);
 
   parser_.SetRecursionLimit(101);
   ExpectSuccessAndTree(input, &message, nullptr);
 }
 
+TEST_F(TextFormatParserTest, SetRecursionLimitUnknownFieldValue) {
+  const char* format = "[$0]";
+  std::string input = "\"test_value\"";
+  for (int i = 0; i < 99; ++i) input = strings::Substitute(format, input);
+  std::string not_deep_input = StrCat("unknown_nested_array: ", input);
+
+  parser_.AllowUnknownField(true);
+  parser_.SetRecursionLimit(100);
+
+  unittest::NestedTestAllTypes message;
+  ExpectSuccessAndTree(not_deep_input, &message, nullptr);
+
+  input = strings::Substitute(format, input);
+  std::string deep_input = StrCat("unknown_nested_array: ", input);
+  ExpectMessage(
+      deep_input,
+      "WARNING:Message type \"protobuf_unittest.NestedTestAllTypes\" has no "
+      "field named \"unknown_nested_array\".\n1:123: Message is too deep, the "
+      "parser exceeded the configured recursion limit of 100.",
+      1, 21, &message, false);
+
+  parser_.SetRecursionLimit(101);
+  ExpectSuccessAndTree(deep_input, &message, nullptr);
+}
+
+TEST_F(TextFormatParserTest, SetRecursionLimitUnknownFieldMessage) {
+  const char* format = "unknown_child: { $0 }";
+  std::string input;
+  for (int i = 0; i < 100; ++i) input = strings::Substitute(format, input);
+
+  parser_.AllowUnknownField(true);
+  parser_.SetRecursionLimit(100);
+
+  unittest::NestedTestAllTypes message;
+  ExpectSuccessAndTree(input, &message, nullptr);
+
+  input = strings::Substitute(format, input);
+  ExpectMessage(
+      input,
+      "WARNING:Message type \"protobuf_unittest.NestedTestAllTypes\" has no "
+      "field named \"unknown_child\".\n1:1716: Message is too deep, the parser "
+      "exceeded the configured recursion limit of 100.",
+      1, 14, &message, false);
+
+  parser_.SetRecursionLimit(101);
+  ExpectSuccessAndTree(input, &message, nullptr);
+}
+
+TEST_F(TextFormatParserTest, ParseAnyFieldWithAdditionalWhiteSpaces) {
+  Any any;
+  std::string parse_string =
+      "[type.googleapis.com/protobuf_unittest.TestAllTypes] \t :  \t {\n"
+      "  optional_int32: 321\n"
+      "  optional_string: \"teststr0\"\n"
+      "}\n";
+
+  ASSERT_TRUE(TextFormat::ParseFromString(parse_string, &any));
+
+  TextFormat::Printer printer;
+  printer.SetExpandAny(true);
+  std::string text;
+  ASSERT_TRUE(printer.PrintToString(any, &text));
+  EXPECT_EQ(text,
+            "[type.googleapis.com/protobuf_unittest.TestAllTypes] {\n"
+            "  optional_int32: 321\n"
+            "  optional_string: \"teststr0\"\n"
+            "}\n");
+}
+
+TEST_F(TextFormatParserTest, ParseExtensionFieldWithAdditionalWhiteSpaces) {
+  unittest::TestAllExtensions proto;
+  std::string parse_string =
+      "[protobuf_unittest.optional_int32_extension]   : \t 101\n"
+      "[protobuf_unittest.optional_int64_extension] \t : 102\n";
+
+  ASSERT_TRUE(TextFormat::ParseFromString(parse_string, &proto));
+
+  TextFormat::Printer printer;
+  std::string text;
+  ASSERT_TRUE(printer.PrintToString(proto, &text));
+  EXPECT_EQ(text,
+            "[protobuf_unittest.optional_int32_extension]: 101\n"
+            "[protobuf_unittest.optional_int64_extension]: 102\n");
+}
+
+TEST_F(TextFormatParserTest, ParseNormalFieldWithAdditionalWhiteSpaces) {
+  unittest::TestAllTypes proto;
+  std::string parse_string =
+      "repeated_int32  : \t 1\n"
+      "repeated_int32: 2\n"
+      "repeated_nested_message: {\n"
+      "  bb: 3\n"
+      "}\n"
+      "repeated_nested_message  : \t {\n"
+      "  bb: 4\n"
+      "}\n"
+      "repeated_nested_message     {\n"
+      "  bb: 5\n"
+      "}\n";
+
+  ASSERT_TRUE(TextFormat::ParseFromString(parse_string, &proto));
+
+  TextFormat::Printer printer;
+  std::string text;
+  ASSERT_TRUE(printer.PrintToString(proto, &text));
+  EXPECT_EQ(text,
+            "repeated_int32: 1\n"
+            "repeated_int32: 2\n"
+            "repeated_nested_message {\n"
+            "  bb: 3\n"
+            "}\n"
+            "repeated_nested_message {\n"
+            "  bb: 4\n"
+            "}\n"
+            "repeated_nested_message {\n"
+            "  bb: 5\n"
+            "}\n");
+}
+
+TEST_F(TextFormatParserTest, ParseSkippedFieldWithAdditionalWhiteSpaces) {
+  protobuf_unittest::TestAllTypes proto;
+  TextFormat::Parser parser;
+  parser.AllowUnknownField(true);
+  EXPECT_TRUE(
+      parser.ParseFromString("optional_int32: 321\n"
+                             "unknown_field1   : \t 12345\n"
+                             "[somewhere.unknown_extension1]   {\n"
+                             "  unknown_field2 \t :   12345\n"
+                             "}\n"
+                             "[somewhere.unknown_extension2]    : \t {\n"
+                             "  unknown_field3     \t :   12345\n"
+                             "  [somewhere.unknown_extension3]    \t :   {\n"
+                             "    unknown_field4:   10\n"
+                             "  }\n"
+                             "  [somewhere.unknown_extension4] \t {\n"
+                             "  }\n"
+                             "}\n",
+                             &proto));
+  std::string text;
+  TextFormat::Printer printer;
+  ASSERT_TRUE(printer.PrintToString(proto, &text));
+  EXPECT_EQ(text, "optional_int32: 321\n");
+}
+
 class TextFormatMessageSetTest : public testing::Test {
  protected:
-  static const char proto_debug_string_[];
+  static const char proto_text_format_[];
 };
-const char TextFormatMessageSetTest::proto_debug_string_[] =
+const char TextFormatMessageSetTest::proto_text_format_[] =
     "message_set {\n"
     "  [protobuf_unittest.TestMessageSetExtension1] {\n"
     "    i: 23\n"
@@ -1941,12 +2088,14 @@ TEST_F(TextFormatMessageSetTest, Serialize) {
       proto.mutable_message_set()->MutableExtension(
           protobuf_unittest::TestMessageSetExtension2::message_set_extension);
   item_b->set_str("foo");
-  EXPECT_EQ(proto_debug_string_, proto.DebugString());
+  std::string actual_proto_text_format;
+  TextFormat::PrintToString(proto, &actual_proto_text_format);
+  EXPECT_EQ(proto_text_format_, actual_proto_text_format);
 }
 
 TEST_F(TextFormatMessageSetTest, Deserialize) {
   protobuf_unittest::TestMessageSetContainer proto;
-  ASSERT_TRUE(TextFormat::ParseFromString(proto_debug_string_, &proto));
+  ASSERT_TRUE(TextFormat::ParseFromString(proto_text_format_, &proto));
   EXPECT_EQ(
       23,
       proto.message_set()
@@ -2005,7 +2154,7 @@ TEST(TextFormatUnknownFieldTest, TestUnknownField) {
                              "  }\n"
                              ">",
                              &proto));
-  // Unmatched delimeters for message body
+  // Unmatched delimiters for message body
   EXPECT_FALSE(parser.ParseFromString("unknown_message: {>", &proto));
   // Unknown extension
   EXPECT_TRUE(
@@ -2106,7 +2255,26 @@ TEST(TextFormatUnknownFieldTest, TestUnknownExtension) {
   EXPECT_FALSE(parser.ParseFromString("unknown_field: 1", &proto));
 }
 
+TEST(TextFormatFloatingPointTest, PreservesNegative0) {
+  proto3_unittest::TestAllTypes in_message;
+  in_message.set_optional_float(-0.0f);
+  in_message.set_optional_double(-0.0);
+  TextFormat::Printer printer;
+  std::string serialized;
+  EXPECT_TRUE(printer.PrintToString(in_message, &serialized));
+  proto3_unittest::TestAllTypes out_message;
+  TextFormat::Parser parser;
+  EXPECT_TRUE(parser.ParseFromString(serialized, &out_message));
+  EXPECT_EQ(in_message.optional_float(), out_message.optional_float());
+  EXPECT_EQ(std::signbit(in_message.optional_float()),
+            std::signbit(out_message.optional_float()));
+  EXPECT_EQ(in_message.optional_double(), out_message.optional_double());
+  EXPECT_EQ(std::signbit(in_message.optional_double()),
+            std::signbit(out_message.optional_double()));
+}
 
 }  // namespace text_format_unittest
 }  // namespace protobuf
 }  // namespace google
+
+#include <google/protobuf/port_undef.inc>
