@@ -32,16 +32,13 @@
 
 #include <google/protobuf/util/field_comparator.h>
 
+#include <limits>
+
 #include <google/protobuf/unittest.pb.h>
 #include <google/protobuf/descriptor.h>
-#include <google/protobuf/stubs/mathutil.h>
-// This gtest header is put after mathutil.h intentionally. We have to do
-// this because mathutil.h includes mathlimits.h which requires cmath not
-// being included to compile on some versions of gcc:
-//   https://github.com/google/protobuf/blob/818c5eee08840355d70d2f3bdf1a2f17986a5e70/src/google/protobuf/stubs/mathlimits.h#L48
-// and the opensource version gtest.h header includes cmath transitively
-// somehow.
 #include <gtest/gtest.h>
+#include <google/protobuf/stubs/mathutil.h>
+
 namespace google {
 namespace protobuf {
 namespace util {
@@ -51,9 +48,7 @@ using protobuf_unittest::TestAllTypes;
 
 class DefaultFieldComparatorTest : public ::testing::Test {
  protected:
-  void SetUp() {
-    descriptor_ = TestAllTypes::descriptor();
-  }
+  void SetUp() override { descriptor_ = TestAllTypes::descriptor(); }
 
   const Descriptor* descriptor_;
   DefaultFieldComparator comparator_;
@@ -62,24 +57,26 @@ class DefaultFieldComparatorTest : public ::testing::Test {
 };
 
 TEST_F(DefaultFieldComparatorTest, RecursesIntoGroup) {
-  const FieldDescriptor* field =
-      descriptor_->FindFieldByName("optionalgroup");
-  EXPECT_EQ(FieldComparator::RECURSE,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  const FieldDescriptor* field = descriptor_->FindFieldByName("optionalgroup");
+  EXPECT_EQ(
+      FieldComparator::RECURSE,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, RecursesIntoNestedMessage) {
   const FieldDescriptor* field =
       descriptor_->FindFieldByName("optional_nested_message");
-  EXPECT_EQ(FieldComparator::RECURSE,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::RECURSE,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, RecursesIntoForeignMessage) {
   const FieldDescriptor* field =
       descriptor_->FindFieldByName("optional_foreign_message");
-  EXPECT_EQ(FieldComparator::RECURSE,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::RECURSE,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, Int32Comparison) {
@@ -87,12 +84,13 @@ TEST_F(DefaultFieldComparatorTest, Int32Comparison) {
   message_1_.set_optional_int32(1);
   message_2_.set_optional_int32(1);
 
-  EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(FieldComparator::SAME, comparator_.Compare(message_1_, message_2_,
+                                                       field, -1, -1, nullptr));
 
   message_2_.set_optional_int32(-1);
-  EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::DIFFERENT,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, Int64Comparison) {
@@ -100,12 +98,13 @@ TEST_F(DefaultFieldComparatorTest, Int64Comparison) {
   message_1_.set_optional_int64(1L);
   message_2_.set_optional_int64(1L);
 
-  EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(FieldComparator::SAME, comparator_.Compare(message_1_, message_2_,
+                                                       field, -1, -1, nullptr));
 
   message_2_.set_optional_int64(-1L);
-  EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::DIFFERENT,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, UInt32Comparison) {
@@ -114,12 +113,13 @@ TEST_F(DefaultFieldComparatorTest, UInt32Comparison) {
   message_1_.set_optional_uint32(1);
   message_2_.set_optional_uint32(1);
 
-  EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(FieldComparator::SAME, comparator_.Compare(message_1_, message_2_,
+                                                       field, -1, -1, nullptr));
 
   message_2_.set_optional_uint32(2);
-  EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::DIFFERENT,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, UInt64Comparison) {
@@ -128,26 +128,27 @@ TEST_F(DefaultFieldComparatorTest, UInt64Comparison) {
   message_1_.set_optional_uint64(1L);
   message_2_.set_optional_uint64(1L);
 
-  EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(FieldComparator::SAME, comparator_.Compare(message_1_, message_2_,
+                                                       field, -1, -1, nullptr));
 
   message_2_.set_optional_uint64(2L);
-  EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::DIFFERENT,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, BooleanComparison) {
-  const FieldDescriptor* field =
-      descriptor_->FindFieldByName("optional_bool");
+  const FieldDescriptor* field = descriptor_->FindFieldByName("optional_bool");
   message_1_.set_optional_bool(true);
   message_2_.set_optional_bool(true);
 
-  EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(FieldComparator::SAME, comparator_.Compare(message_1_, message_2_,
+                                                       field, -1, -1, nullptr));
 
   message_2_.set_optional_bool(false);
-  EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::DIFFERENT,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, EnumComparison) {
@@ -156,12 +157,13 @@ TEST_F(DefaultFieldComparatorTest, EnumComparison) {
   message_1_.set_optional_nested_enum(TestAllTypes::BAR);
   message_2_.set_optional_nested_enum(TestAllTypes::BAR);
 
-  EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(FieldComparator::SAME, comparator_.Compare(message_1_, message_2_,
+                                                       field, -1, -1, nullptr));
 
   message_2_.set_optional_nested_enum(TestAllTypes::BAZ);
-  EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::DIFFERENT,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, StringComparison) {
@@ -170,12 +172,13 @@ TEST_F(DefaultFieldComparatorTest, StringComparison) {
   message_1_.set_optional_string("foo");
   message_2_.set_optional_string("foo");
 
-  EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(FieldComparator::SAME, comparator_.Compare(message_1_, message_2_,
+                                                       field, -1, -1, nullptr));
 
   message_2_.set_optional_string("bar");
-  EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, -1, -1, NULL));
+  EXPECT_EQ(
+      FieldComparator::DIFFERENT,
+      comparator_.Compare(message_1_, message_2_, field, -1, -1, nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, FloatingPointComparisonExact) {
@@ -190,21 +193,21 @@ TEST_F(DefaultFieldComparatorTest, FloatingPointComparisonExact) {
   message_2_.set_optional_double(0.1);
 
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   message_2_.set_optional_float(0.2f);
   message_2_.set_optional_double(0.2);
 
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, FloatingPointComparisonApproximate) {
@@ -230,20 +233,20 @@ TEST_F(DefaultFieldComparatorTest, FloatingPointComparisonApproximate) {
   // DefaultFieldComparator's default float comparison mode is EXACT.
   ASSERT_EQ(DefaultFieldComparator::EXACT, comparator_.float_comparison());
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   comparator_.set_float_comparison(DefaultFieldComparator::APPROXIMATE);
 
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest, FloatingPointComparisonTreatNaNsAsEqual) {
@@ -252,45 +255,45 @@ TEST_F(DefaultFieldComparatorTest, FloatingPointComparisonTreatNaNsAsEqual) {
   const FieldDescriptor* field_double =
       descriptor_->FindFieldByName("optional_double");
 
-  message_1_.set_optional_float(MathLimits<float>::kNaN);
-  message_2_.set_optional_float(MathLimits<float>::kNaN);
-  message_1_.set_optional_double(MathLimits<double>::kNaN);
-  message_2_.set_optional_double(MathLimits<double>::kNaN);
+  message_1_.set_optional_float(std::numeric_limits<float>::quiet_NaN());
+  message_2_.set_optional_float(std::numeric_limits<float>::quiet_NaN());
+  message_1_.set_optional_double(std::numeric_limits<double>::quiet_NaN());
+  message_2_.set_optional_double(std::numeric_limits<double>::quiet_NaN());
 
   // DefaultFieldComparator's default float comparison mode is EXACT with
   // treating NaNs as different.
   ASSERT_EQ(DefaultFieldComparator::EXACT, comparator_.float_comparison());
   ASSERT_EQ(false, comparator_.treat_nan_as_equal());
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
   comparator_.set_float_comparison(DefaultFieldComparator::APPROXIMATE);
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   comparator_.set_treat_nan_as_equal(true);
   ASSERT_EQ(true, comparator_.treat_nan_as_equal());
   comparator_.set_float_comparison(DefaultFieldComparator::EXACT);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
   comparator_.set_float_comparison(DefaultFieldComparator::APPROXIMATE);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest,
@@ -307,61 +310,61 @@ TEST_F(DefaultFieldComparatorTest,
 
   comparator_.set_float_comparison(DefaultFieldComparator::APPROXIMATE);
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Should fail since the fraction is too low.
   comparator_.SetFractionAndMargin(field_float, 0.01, 0.0);
   comparator_.SetFractionAndMargin(field_double, 0.01, 0.0);
 
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Should fail since the margin is too low.
   comparator_.SetFractionAndMargin(field_float, 0.0, 9.0);
   comparator_.SetFractionAndMargin(field_double, 0.0, 9.0);
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Should succeed since the fraction is high enough.
   comparator_.SetFractionAndMargin(field_float, 0.2, 0.0);
   comparator_.SetFractionAndMargin(field_double, 0.2, 0.0);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Should succeed since the margin is high enough.
   comparator_.SetFractionAndMargin(field_float, 0.0, 10.0);
   comparator_.SetFractionAndMargin(field_double, 0.0, 10.0);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Setting values for one of the fields should not affect the other.
   comparator_.SetFractionAndMargin(field_double, 0.0, 0.0);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // +inf should be equal even though they are not technically within margin or
   // fraction.
@@ -372,11 +375,11 @@ TEST_F(DefaultFieldComparatorTest,
   comparator_.SetFractionAndMargin(field_float, 0.0, 0.0);
   comparator_.SetFractionAndMargin(field_double, 0.0, 0.0);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // -inf should be equal even though they are not technically within margin or
   // fraction.
@@ -387,11 +390,25 @@ TEST_F(DefaultFieldComparatorTest,
   comparator_.SetFractionAndMargin(field_float, 0.0, 0.0);
   comparator_.SetFractionAndMargin(field_double, 0.0, 0.0);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
+
+  // Finite values and inf should not be equal, even for a positive fraction.
+  message_1_.set_optional_float(std::numeric_limits<float>::infinity());
+  message_2_.set_optional_float(0.0f);
+  message_1_.set_optional_double(std::numeric_limits<double>::infinity());
+  message_2_.set_optional_double(0.0);
+  comparator_.SetFractionAndMargin(field_float, 0.1, 0.0);
+  comparator_.SetFractionAndMargin(field_double, 0.1, 0.0);
+  EXPECT_EQ(FieldComparator::DIFFERENT,
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
+  EXPECT_EQ(FieldComparator::DIFFERENT,
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 }
 
 TEST_F(DefaultFieldComparatorTest,
@@ -408,22 +425,22 @@ TEST_F(DefaultFieldComparatorTest,
 
   comparator_.set_float_comparison(DefaultFieldComparator::APPROXIMATE);
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Set default fraction and margin.
   comparator_.SetDefaultFractionAndMargin(0.01, 0.0);
 
   // Float comparisons should fail since the fraction is too low.
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Set field-specific fraction and margin for one field (field_float) but not
   // the other (field_double)
@@ -432,36 +449,36 @@ TEST_F(DefaultFieldComparatorTest,
   // The field with the override should succeed, since its field-specific
   // fraction is high enough.
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   // The field with no override should fail, since the default fraction is too
   // low
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // Set the default fraction and margin high enough so that fields that use
   // the default should succeed
   comparator_.SetDefaultFractionAndMargin(0.2, 0.0);
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 
   // The field with an override should still be OK
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
 
   // Set fraction and margin for the field with an override to be too low
   comparator_.SetFractionAndMargin(field_float, 0.01, 0.0);
 
   // Now our default is high enough but field_float's override is too low.
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_,
-                                field_float, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_float, -1, -1,
+                                nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_,
-                                field_double, -1, -1, NULL));
+            comparator_.Compare(message_1_, message_2_, field_double, -1, -1,
+                                nullptr));
 }
 
 // Simple test checking whether we compare values at correct indices.
@@ -475,14 +492,14 @@ TEST_F(DefaultFieldComparatorTest, RepeatedFieldComparison) {
   message_2_.add_repeated_string("baz");
 
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, 0, 0, NULL));
+            comparator_.Compare(message_1_, message_2_, field, 0, 0, nullptr));
   EXPECT_EQ(FieldComparator::DIFFERENT,
-            comparator_.Compare(message_1_, message_2_, field, 1, 1, NULL));
+            comparator_.Compare(message_1_, message_2_, field, 1, 1, nullptr));
   EXPECT_EQ(FieldComparator::SAME,
-            comparator_.Compare(message_1_, message_2_, field, 1, 0, NULL));
+            comparator_.Compare(message_1_, message_2_, field, 1, 0, nullptr));
 }
 
+}  // namespace
 }  // namespace util
 }  // namespace protobuf
-}  // namespace
 }  // namespace google
